<?php
/**
 *
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Services
 */

/**
 * Data Definition stores, updates, and retrieves data definitions.
 * 
 * @author Phil Calder, Matthew Ross
 * @package Services
 */
class DataDefinition extends Service {
	
	/**
	 * @access private
	 */
	var $table = '';
	/**
	 * @access private
	 */
	var $cache = array();
	/**
	 * @access private
	 */
	var $fields_table = '';
	/**
	 * @access private
	 */
	var $data_options_table = '';
	/**
	 * @access private
	 */
	var $webmat_dropdown_values_cache = array();

	function DataDefinition($table = '') {
		global $SERVICES;
		
		$this->db =& $SERVICES->getService('Database');
		
		$this->fields_table = 'SystemDataDefinition__fields';
		$this->data_options_table = 'SystemDataDefinition__data_options';
		
		if (!empty($table)) {
			$this->setTable($table);
		}
		
		$this->log =& $SERVICES->getService('Log');
		
	}
	
	function setTable($table) {
		$this->table = $table;
		// $this->cache = array(); // Removed - we now store cache for each table (used to reset fields and data_options)
	}
	
	
	function getTable() {
		return $this->table;
	}
	
	/**
	 * @param string $table_name
	 * @return array $data_definition The data definition for the given table
	 */
	function getDataDefinition($table = '') {
		if (!empty($table)) {
			$this->setTable($table);	
		}
		
		$data_definition = array();
		
		$fields = $this->getFields($table);
		$options = $this->getOptions($table);
		
		return array('fields'=>$fields, 'data_options'=>$options);
	}
	
	function getFieldNames($table, $include_foreign_modules = true, $module = false, $include_labels = false) {
		// Note: Written for Module builder multi select (data_options values)
		// Phil 20061201 Added second param $include_foreign_modules (for Export)
		
		
		$fields = $this->getFields($table);
		
		$field_names = array();
		if (is_array($fields)) {
			foreach ($fields as $k => $v) {
				if (isset($v['field_key']) && $v['field_key']) {
					if ($include_labels) {
						$field_names[] = array(
							'field_key' => $v['field_key'], 
							'label' => $v['label']
						);
					} else {
						$field_names[] = $v['field_key'];
					}
				}
			}
		}
		
		if ($module && ($include_foreign_modules === true)) {
			$ff = $this->getForeignFields($module, $table);
			if (is_array($ff)) {
				foreach ($fields as $k => $v) {
					
					if (isset($v['field_key']) && $v['field_key']) {
						// TODO // print '<div>k=' . $k . '</div>'; // print_r($v); print '<hr>';
						if ($include_labels) {
							$field_names[] = array(
								'field_key' => $v['field_key'], 
								'label' => $v['label']
							);
						} else {
							$field_names[] = $v['field_key'];
						}
					}
				}
			}
			// print '<div>ff:</div>'; print_r($ff); print '<hr>';
		}
		return $field_names;
	}
	
	/**
	 * Returns the number of fields listed in the data definition for a given table.
	 *
	 * @return integer number of fields
	 * See WebMAT documentation for a description of the fields array.
	 */
	function countFields($table = '') {
		// $this->printDebug('AdminDataDefinition::countFields table ' . $table);
		
		if (!empty($table)) {
			$this->setTable($table);	
		}
		
		$result = $this->db->quickGet(
			'count(*)', 
			$this->fields_table,
			// 'field_table LIKE "' . $this->getTable() . '"'
			'field_table = "' . $this->getTable() . '"' // Phil 20051004
		);
	}
	
	/**
	 * Returns the field definitions for use in a data dictionary.
	 *
	 * @return array of field info, numeric key, value is an array
	 * See WebMAT documentation for a description of the fields array.
	 */
	function getFields($table = '', $limit = '', $sort = '', $custom_only = false) {
		// $this->printDebug('AdminDataDefinition::getFields table ' . $table);
		$sort_str = '';
		
		if (!empty($table)) {
			$this->setTable($table);	
		}
		
		if ($custom_only) {
			bcrm_unset($this->cache, $this->getTable());
		}
		
		//MCR 20051021 added strtolower
		$standard = ((strtolower(get_class($this)) == 'datadefinition') ? true : false);
		//$standard = ((get_class($this) == 'datadefinition') ? true : false);
		
		if (is_array($sort) && isset($sort['field']) && isset($sort['order'])) {
			$sort_str = $sort['field'] . ' ' . $sort['order'];
			$sort_str = (empty($sort_str) ? '' : $sort_str . ', ');
		}
		
		if ($standard) {
			$order = 'field_priority ASC, field_id ASC'; // in the order they were created
		} else {
			$order = 'field_is_custom ASC'; // standard fields first
		}
		
		$order = $sort_str . $order;
		// $this->printDebug('DataDefinition::getFields order by ' . $order);
		
		if (!isset($this->cache[$this->getTable()]['fields']) 
		|| !is_array($this->cache[$this->getTable()]['fields'])) {
			
			// $this->printDebug('DataDefinition::getFields() - not cached, getting from database');
			$this->cache[$this->getTable()]['fields'] = array();
			
			// $where = 'field_table LIKE "' . $this->getTable() . '"'; // AND module = "' . $this->module . '"'
			$where = 'field_table = "' . $this->getTable() . '"'; // Phil 20051004
			
			if ($custom_only) {
				$where .= ' AND ( (`field_is_custom`=1) OR (`field_is_custom`=2) OR (`field_key`="id") )';	
			}
			
			$result = $this->db->select(
				'*', 
				$this->fields_table,
				$where, 
				$order, 
				'', 
				$limit
			);
			
			if ($this->db->hasError()) {
				$this->db->printError();
			}
			
			while($row = $this->db->getRow($result)) {
				if ($standard) { // The admin extension of this class can use these fields
					bcrm_unset($row, 'field_id');
					//bcrm_unset($row, 'field_is_custom');//syed 20050626
					bcrm_unset($row, 'field_table');
				}
				
				// Phil - removed this from the 'if' block above
				// Unsets all '0' or '' values
				foreach($row as $k => $v) {
					if (empty($v)) {
						// $this->printDebug('unsetting ' . $k);
						bcrm_unset($row, $k);
					}
					
					if ( ($k == 'label') || ($k == 'list_label') || ($k == 'explanation')) {
						$row[$k] = $this->processLanguageText($v); // Phil 20041217
						// print $k . '=' . $v . ' = ' . $row[$k] . '<br>';
					}
				}
				
				// $this->cache[$this->getTable()]['fields'][] = $row;
				$key = $row['field_key']; // Phil 20050302 - Added field_key index (for overriding fields in a data_definition text file)
				$this->cache[$this->getTable()]['fields'][$key] = $row;
			}
		}
		// print_r($this->cache[$this->getTable()]['fields']); print '<hr>';
		return $this->cache[$this->getTable()]['fields'];
	}
	
	/**
	 * Returns the custom field definitions for use in a data dictionary.
	 *
	 * @return array of field info, numeric key, value is an array
	 * See WebMAT documentation for a description of the fields array.
	 */
	function getCustomFields($table = '', $limit = '', $sort = '') {
		if (substr($table, strlen($table) - strlen($this->custom_suffix)) == $this->custom_suffix) {
			$table = substr($table, 0, (strlen($table) - strlen($this->custom_suffix)) );
		}
		$data_definition = $this->getFields($table, $limit, $sort, true);
		bcrm_unset($this->cache, $this->getTable());
		// print '<div>getCustomFields for ' . $table . '</div>'; print_r($data_definition); print '<hr>';
		return $data_definition;
	}
	
	/**
	 * Returns the data options array (for a data_definition) for the table set by $this->setTable
	 *
	 * @return array $data_definition, possible keys are
	 * 'method', either include or exclude.
	 * 'values', an array of field names.
	 * 'options', an array with optional keys:
	 *         'exclude_hidden' 0 or 1
	 *         'exclude_calcualted' 0 or 1
	 *	       'event_date' a field key
	 *	       'no_trim' 0 or 1
	 * see the WebMAT documentation for a complete description.
	 */
	function getOptions($table = '') {
		// $this->printDebug('AdminDataDefinition::getOptions table ' . $table);
		if (!empty($table)) {
			$this->setTable($table);	
		}
		
		$fields = array();
		
		if (!isset($this->cache[$this->getTable()]['data_options']) 
		|| !is_array($this->cache[$this->getTable()]['data_options'])) {

			// $this->printDebug('DataDefinition::getOptions() - not cached, getting from database');
			$result = $this->db->select(
				'*', 
				$this->data_options_table,
				// 'active=1 AND field_table LIKE "' . $this->getTable() . '"' // AND module = "' . $this->module . '"'
				'active=1 AND field_table = "' . $this->getTable() . '"' // Phil 20051004
			);
			
			while($row = $this->db->getRow($result)) {
				// print_r($row); print '<hr>';
				bcrm_unset($row, 'id');
				// bcrm_unset($row, 'field_is_custom');
				bcrm_unset($row, 'field_table');
				
				$item = array();
				// $item['method'] = $row['method'];
				if (isset($row['method']) && $row['method']) {
					$item['method'] = $row['method'];
				}
				
				if (isset($row['option_values']) && $row['option_values']) {
					$item['values'] = explode(',', $row['option_values']);
					bcrm_unset($row, 'option_values');
				}
				
				$item['options'] = array();
				if (isset($row['exclude_hidden']) && $row['exclude_hidden']) { 
					$item['options']['exclude_hidden'] = $row['exclude_hidden']; 
				}
				if (isset($row['exclude_calculated']) && $row['exclude_calculated']) { 
					$item['options']['exclude_calculated'] = $row['exclude_calculated']; 
				}
				if (isset($row['dropdown_values']) && $row['dropdown_values']) { 
					$item['options']['dropdown_values'] = $row['dropdown_values']; 
				}
				// if (isset($row['search_title']) && $row['search_title']) { 
					// $item['options']['search_title'] = $row['search_title']; 
				// }
				if (isset($row['event_date']) && $row['event_date']) { 
					$item['options']['event_date'] = $row['event_date']; 
				}
				if (isset($row['no_trim']) && $row['no_trim']) { 
					$item['options']['no_trim'] = $row['no_trim']; 
				}
				
				if (!sizeof($item['options'])) {
					bcrm_unset($item, 'options');
				}
				
				bcrm_unset($row, 'active');
				$this->cache[$this->getTable()]['data_options'][$row['type']] = $item;
			}
		}
		return $this->cache[$this->getTable()]['data_options'];
	}
	
	function getOption($id, $table = '') {
		// $this->printDebug('AdminDataDefinition::getOption id ' . $id . ', table ' . $table);
		if (!empty($table)) {
			$this->setTable($table);	
		}
		
		$fields = array();
		// $this->printDebug('DataDefinition::getOption(' . $id . ')');
		
		$result = $this->db->quickGet(
			'*', 
			$this->data_options_table,
			'id=' . $id
		);
		
		if (is_array($result) && isset($result['option_values'])) {
			$result['values'] = $result['option_values'];
			bcrm_unset($result, 'option_values');
		}
		return $result;
	}
	
	function getOptionByType($type, $table = '') {
		if (!empty($table)) {
			$this->setTable($table);	
		}
		
		$fields = array();
		// $this->printDebug('DataDefinition::getOptionByType(' . $id . ')');
		
		$result = $this->db->quickGet(
			'*', 
			$this->data_options_table,
			'type="' . $type . '" AND field_table = "' . $table . '"'
		);
		// print '<p>result:'; print_r($result); print '</p>';
		return $result;
	}
	
	function processLanguageText($label) {
		$label = stripslashes($label);
						
		if (substr($label, 0, 2) == '__') {
			global $L, $SERVICES;
			$SERVICES->getService('Language');
			// print '<p>1- has prefix: ' . $label . ' [' . substr($label, 2) . ']</p>';
			$label = $SERVICES->language->getLanguageText(substr($label, 2));
			// print '<p>2- processed: ' . $label . '</p>';
		}
		return $label;
	}
	
	/**
	 * Dropdown values stored / retrieved from cache
	 *
	 * @see WebMATDataDictionary::getDropdownValues
	 */
	function getWebMATCachedDropdownValues($module, $table, $field = 'name') {
		if (
			is_array($this->webmat_dropdown_values_cache) && 
			isset($this->webmat_dropdown_values_cache[$module]) && 
			is_array($this->webmat_dropdown_values_cache[$module]) && 
			isset($this->webmat_dropdown_values_cache[$module][$table]) && 
			is_array($this->webmat_dropdown_values_cache[$module][$table]) && 
			isset($this->webmat_dropdown_values_cache[$module][$table][$field]) && 
			is_array($this->webmat_dropdown_values_cache[$module][$table][$field])
		) {
			// print '<div>getting cached dropdown values for ' . $module . ' ' . $table . ' ' . $field . '</div>';
			// print_r($this->webmat_dropdown_values_cache[$module][$table][$field]); print '<hr />';
			return $this->webmat_dropdown_values_cache[$module][$table][$field];
		}
		return false;
	}
	
	/**
	 * Dropdown values stored / retrieved from cache
	 *
	 * @see WebMATDataDictionary::getDropdownValues
	 */
	function setWebMATCachedDropdownValues($dropdown_values, $module, $table, $field = 'name') {
		// print '<div>setting cached dropdown values for ' . $module . ' ' . $table . ' ' . $field . '</div>';
		$this->webmat_dropdown_values_cache[$module][$table][$field] = (array)$dropdown_values;
	}
	
} // class DataDefinition

/**
 * Admin Data Definition provides extended functionality to create new data definitions.
 *
 * Database tables and fields are added to match the new data definitions.
 * @author Phil Calder
 * @package Services
 */
class AdminDataDefinition extends DataDefinition{
	/**
	 * @access private
	 */
	var $custom_suffix;
	/**
	 * @access private
	 */
	var $seen_table_suffix;
	/**
	 * @access private
	 */
	var $system_dropdowns = array();
	
	function AdminDataDefinition($table = '') {
		global $SERVICES;
		
		// Only 'Admin' module admistrators can use this class
		$ua =& $SERVICES->getService('Authentication');
		$ua->adminRestrict('Admin');
		
		$SERVICES->getService('Language');
		$SERVICES->language->getLanguage('Admin');
		
		$this->custom_suffix = '_custom';
		$this->seen_table_suffix = '_seen';
		$this->dropdown_keyword = 'dropdown';
		
		$this->system_dropdowns = array(
			'__USERS__', 
			'__USERSALL__', // Phil 20060814
			'__GROUPS__', 
			'__TEAMGROUPS__', 
			'__PRIVSGROUPS__', 
			'__PRIVATEGROUPS__'
		);
		parent::DataDefinition();
	}
	
	/**
	 * Returns the number of data options for all tables
	 *
	 * @return integer number of data options.
	 */
	function countAllOptions($table) {
		// $this->printDebug('AdminDataDefinition::countAllOptions(' . $table . ')');
		$items = array();
		$where = (!empty($table) ? 'field_table = "'. $table . '"' : '');
		
		$result = $this->db->quickGet(
			'count(*)', 
			$this->data_options_table,
			$where
		);
		return $result;
	}	
	
	/**
	 * Returns the data options arrays for all tables
	 *
	 * @return array $data_definitions, contains arrays with possible keys:
	 * 'method', either include or exclude.
	 * 'values', an array of field names.
	 * 'options', an array with optional keys:
	 *         'exclude_hidden' 0 or 1
	 *         'exclude_calcualted' 0 or 1
	 *	       'event_date' a field key
	 *	       'no_trim' 0 or 1
	 * see the WebMAT documentation for a complete description.
	 */
	function getAllOptions($table = '', $limit = '', $sort = '') {
		$items = array();
		$where = $order = '';
		
		if (!empty($table)) {
			$where = 'field_table = "' . $table . '"';
		}
		
		if ($sort['field']=='values') {
			// $sort['field'] = '`values`'; //MySQL keyword	
			$sort['field'] = 'option_values';
		}
		
		if (is_array($sort)) {
			$order = $sort['field'] . ' ' . $sort['order'];
		}
		// $this->printDebug('AdminDataDefinition::getAllOptions(' . $limit . ')');
		
		$result = $this->db->select(
			'*', 
			$this->data_options_table, 
			$where, 
			$order, // 'field_table ASC',
			'',
			$limit
		);
		
		while($row = $this->db->getRow($result)) {
			if (isset($row['option_values']) && $row['option_values']) {
				$row['values'] = $row['option_values'];
				bcrm_unset($row, 'option_values');
			}
			$items[] = $row;
		}
		
		return $items;
	}	
	
	function setDataOptions($item) {
		// $this->printDebug('AdminDataDefinition::setDataOptions');
		// $this->printDebug($item);
		if (!isset($item['id']) || !$item['id']) {
			return $this->insertDataOption($item);
		}
		return $this->updateDataOption($item);
	}
	
	function updateDataOption($item) {
		// $this->printDebug('AdminDataDefinition::updateDataOption');
		// $this->printDebug($item);
		
		$id = $item['id'];
		bcrm_unset($item, 'id');
		
		// $item['`values`'] = $item['values']; // values is a MySQL keyword
		$item['option_values'] = $item['values']; // values is a MySQL keyword
		bcrm_unset($item, 'values');
		
		if ( ($existing_item = $this->getOption($id)) && is_array($existing_item) ) {
			
			// We only want this to be 'values' for WebMAT
			// since we are comparing the changed data in item we will refer to the existing data as option_values as well
			$existing_item['option_values'] = $existing_item['values']; // values is a MySQL keyword
			bcrm_unset($existing_item, 'values');
			
			// ignore field_table and type - they wont be changing
			// ignore id - wont be in item (we already unset it) and would be different per installation
			// [method] => (blank, include, or exclude)
			// [exclude_calculated] => 0/1 
			// [dropdown_values] => 0/1 
			// [no_trim] => 0/1
			// [active] => 0/1 
			// [values] => (csv string)
			
			// print '<div>updateDataOption</div>'; print_r($existing_item); print '<hr>'; print_r($item); 
			
			$field_table = $item['field_table'];
			$type = $item['type'];
			
			bcrm_unset($item, 'field_table');
			bcrm_unset($item, 'type');
			bcrm_unset($existing_item, 'id');
			bcrm_unset($existing_item, 'field_table');
			bcrm_unset($existing_item, 'type');
			bcrm_unset($existing_item, 'active');
			
			foreach ($existing_item as $k => $v) {
				// print '<div>key:  ' . $k . '</div>';
				// print '<div>1 ' . (isset($item[$k]) ? 'yes' : 'no') . '</div>';
				// print '<div>1a ' . ($item[$k] ? 'yes' : 'no') . '</div>';
				// print '<div>2 ' . (($item[$k] || $existing_item[$k]) ? 'yes' : 'no') . '</div>';
				// print '<div>3 ' . (isset($existing_item[$k]) ? 'yes' : 'no') . '</div>';
				// print '<div>3a ' . ($existing_item[$k] ? 'yes' : 'no') . '</div>';
				// print '<div>4 ' . ( (isset($existing_item[$k]) ? (trim($existing_item[$k]) === trim($item[$k])) : 'yes') ) . '</div>';
				
				// Dont update the following
				if (
					isset($item[$k]) && 
					// ($item[$k] || $existing_item[$k]) && 
					( (isset($existing_item[$k]) ? (trim($existing_item[$k]) === trim($item[$k])) : true) ) 
				) {
					// print '<div>unset ' . $k . '</div>';
					bcrm_unset($existing_item, $k);
					bcrm_unset($item, $k);
				}
				
				// if (isset($existing_item[$k])) {
					// print '
					// <div>
					// ' . $k . ' existing: "' . $existing_item[$k] . '" new: "' . $item[$k] . '"
					// </div>';
				// }
				// print '<hr>';
			}
			// The keys and values remaining in existing_item are added to our new where clause
			// print_r($existing_item);
		}
		
		$where_updated = array();
		if (is_array($existing_item)) {
			foreach ($existing_item as $k => $v) {
				$where_updated[] = "$k='$v'";
			}
		}
		$where_updated = implode(' AND ', $where_updated);
		
		$where = "field_table='" . $field_table . "' AND type='" . $type . "'";
		$where .= (strlen($where_updated) ? ' AND ' . $where_updated : '');
		
		if (is_array($item) && sizeof($item)) {
			// $this->db->log_sql = true;
			$this->db->setSqlLogStatus(true, $field_table);
			$result = $this->db->update(
				$this->data_options_table,
				$item,
				$where
			);
			$this->setLog('updated data option', $field_table . ' ' . $type);
		} else {
			$result = true;	
		}
		
		return $result;
	}
	
	function insertDataOption($item) {
		// $this->printDebug('AdminDataDefinition::insertDataOption');
		// $this->printDebug($item);
		
		// $item['`values`'] = $item['values']; // values is a MySQL keyword
		$item['option_values'] = $item['values']; // values is a MySQL keyword
		bcrm_unset($item, 'values');
		
		// $this->db->log_sql = true;
		$this->db->setSqlLogStatus(true, $item['field_table']);
		
		$result = $this->db->insert(
			$this->data_options_table,
			$item
		);
		
		if (!$result) {
			
		} else {
			$this->setLog('inserted data option', $item['field_table'] . ' ' . $item['type']);
		}
		
		return $result;
	}
	
	// soft-delete
	function deleteDataOption($id) {
		// $this->printDebug('AdminDataDefinition::deleteDataOption with id ' . $id);
		if ($item = $this->getOption($id)) {
			// $this->db->log_sql = true;
			$this->db->setSqlLogStatus(true, $item['field_table']);
			
			$result = $this->db->update(
				$this->data_options_table,
				array('active' => 0),  
				'field_table="' . $item['field_table'] . '" AND type="' . $item['type'] . '"'
			);
			
			if ($result) if ($result) {
				$this->setLog('deleted data option', $item['field_table'] . ' - ' . $item['type']);
			}
			return $result;
		}
	}
	
	function restoreDataOption($id) {
		// $this->printDebug('AdminDataDefinition::restoreDataOption with id ' . $id);
		if ($item = $this->getOption($id)) {
			// $this->db->log_sql = true;
			$this->db->setSqlLogStatus(true, $item['field_table']);
			
			$result = $this->db->update(
				$this->data_options_table,
				array('active' => 1),  
				'field_table="' . $item['field_table'] . '" AND type="' . $item['type'] . '"'
			);
			
			if ($result) {
				$this->setLog('restored data option', $id);
			}
			return $result;
		}
	}
	
	function setItem($item) {
		// $this->printDebug('AdminDataDefinition::setItem');
		// $this->printDebug($item);
		
		// Need a table
		if (!isset($item['field_table']) || !$item['field_table']) {
			// $this->printDebug('AdminDataDefinition::setItem - no field table specified');
			return false;
		}
		
		// Create standard table
		if (!$this->standardTableExists($item['field_table'])) {
			// $this->printDebug('AdminDataDefinition::setItem - table needs to be created: ' . $item['field_table'] . '');
		}
		
		// Create custom table if necessary
		if ( 
		(!isset($item['field_is_custom']) || (isset($item['field_is_custom']) && $item['field_is_custom'] == 1))  
		&& !$this->customTableExists($item['field_table']) 
		&& (!isset($item['foreign']) || (isset($item['foreign']) && !$item['foreign'])) // Phil 20050331
		) {
			
			$this->setLog('AdminDataDefinition::setItem', 'table needs to be created: ' . $this->getCustomTable($item['field_table']));
			
			if (!$this->createCustomTable($item['field_table'])) {
				$this->setLog('AdminDataDefinition::setItem', 'error creating table: ' . $this->getCustomTable($item['field_table']));
				return false;
			}
		}
		
		if (isset($item['dropdown_values'])) {
			$item['dropdown_values'] = $this->setDropdownTableName($item['dropdown_values']);
		}
		
		if (isset($item['display_name']) && $item['display_name']) {
			$this->unsetDisplayNameField($item['field_table']); // Removed display name flag because we are about to change this to a new field
		}
		
		if (isset($item['field_priority']) && (!isset($item['priority_edit']) || !$item['priority_edit'])) { // Phil 20050331
			$item['priority_edit'] = $item['field_priority'];
		}
		
		// new or existing ? (field_id)
		if (isset($item['field_id']) && $item['field_id'] && ($field = $this->getField($item['field_id']))) {
			// $this->printDebug('AdminDataDefinition::setItem - field exists, needs updating: ' . $item['field_id'] . '');
			$retval = $this->updateItem($item);
		} else {
			// $this->printDebug('AdminDataDefinition::setItem - this is a new field');
			$retval = $this->addItem($item);
		}
		
		return $retval;
	}
	
	function addItem($item) {
		// $this->printDebug('AdminDataDefinition::addItem');
		// $this->printDebug($item);
		
		// Must specify a table
		if (!isset($item['field_table']) || !$item['field_table']) {
			// $this->printDebug('AdminDataDefinition::addItem returing false because there was no field_table specified');
			return false;
		}
		
		// field_key must not exist (it would be an 'update' if it did)
		$fields = $this->getFields($item['field_table']);
		// print_r($fields); exit();
		
		foreach($fields as $k =>$v) {
			if ($v['field_key'] == $item['field_key']) {
				// $this->printDebug('AdminDataDefinition::addItem returning false because field_key ' . $item['field_key'] . ' already exists');
				return false;
			}
		}
		
		if (isset($item['foreign'])) { // foreign is a MySQL reserved word
			$item['`foreign`'] = $item['foreign'];
			bcrm_unset($item, 'foreign');
		}
		
		$this->db->log_sql = true;
		$this->db->setSqlLogStatus(true, $item['field_table']);
		
		$_special_date_time = ( (isset($item['_special_date_time']) && $item['_special_date_time']) ? '1' : '0');
		bcrm_unset($item, '_special_date_time');
		
		// Insert the field definition
		$new_field = $this->db->insert(
			$this->fields_table, 
			$item
		);
		
		if ($_special_date_time) {
			$item['_special_date_time'] = 1;
		}

		// --
		$is_not_calculated = (!isset($item['calculated']) || !$item['calculated']);
		if (isset($item['foreign']) && !isset($item['`foreign`'])) {
			$item['`foreign`'] = $item['foreign'];
		}
		bcrm_unset($item, 'foreign');
		$is_not_foreign = (!isset($item['`foreign`']) || !$item['`foreign`']);
		
		if ( $is_not_calculated && $is_not_foreign) {
			// print '<div>Adding field</div>'; print_r($item); print '<hr>';
			if (isset($item['field_is_custom']) && ($item['field_is_custom'] != 1)) {
				$this->addFieldToStandardTable($item);
			} else {
				$this->addFieldToCustomTable($item);
			}
		}
		// $this->printDebug('AdminDataDefinition::addItem returning "new_field" ' . $new_field . '');
		$this->updateDbCache();
		return $new_field;
	}
	
	function updateItem($item) {
		$field_id = $item['field_id'];
		// $where = 'field_id=' . $field_id;
		$field_table = $item['field_table'];
		$field_key = $item['field_key'];
		$where = 'field_table = "' . $field_table . '" AND field_key = "' . $field_key . '"';
		
		// Cannot change these fields
		bcrm_unset($item, 'field_id');
		bcrm_unset($item, 'field_table');
		bcrm_unset($item, 'field_is_custom');
		bcrm_unset($item, 'field_key');
		bcrm_unset($item, 'type');
		
		// dropdown_values
		// default_value
		
		if ( ($existing_item = $this->getField($field_id, true)) && is_array($existing_item) ) {
			bcrm_unset($existing_item, 'field_id');
			bcrm_unset($existing_item, 'field_table');
			bcrm_unset($existing_item, 'field_is_custom');
			bcrm_unset($existing_item, 'field_key');
			bcrm_unset($existing_item, 'type');
			
			// print_r($existing_item); print '<hr>'; print_r($item); 
			
			foreach ($existing_item as $k => $v) {
				// Dont update the following
				if (
					isset($item[$k]) && 
					( (isset($existing_item[$k]) ? (trim($existing_item[$k]) === trim($item[$k])) : true) )
				) {
					// print '<div>unset ' . $k . '</div>';
					bcrm_unset($existing_item, $k);
					bcrm_unset($item, $k);
				}
				
				if (!$item[$k] && !$existing_item[$k]) {
					bcrm_unset($existing_item, $k);
					bcrm_unset($item, $k);
				}
			}
			
			// print '<hr>'; print_r($existing_item); print '<hr>'; print_r($item);
			$where_updated = array();
			if (is_array($existing_item)) {
				foreach ($existing_item as $k => $v) {
					$where_updated[] = "$k='$v'";
				}
			}
			$where_updated = implode(' AND ', $where_updated);
			
			$where .= (strlen($where_updated) ? ' AND ' . $where_updated : '');
			// print '<div>New update:</div>'; print_r($item); print '<div>WHERE: ' . $where . '</div>';
		}
		
		// $this->printDebug('AdminDataDefinition::updateItem (field_id is ' . $field_id . ')');
		// $this->printDebug($item);
		
		if (is_array($item) && sizeof($item)) {
			$this->db->log_sql = true;
			$this->db->setSqlLogStatus(true, $field_table);
			
			return $this->db->update(
				$this->fields_table, 
				$item,
				$where
			);
		}
		
		return true;
	}
	
	/**
	 * There can only be one display name field. This deletes the display name flag from (potentially) all fields in the given table.
	 * Called prior to changing the display name field (ie in setItem (or addItem or updateItem))
	 */
	function unsetDisplayNameField($table_name) {
		// $this->printDebug('AdminDataDefinition::unsetDisplayNameField for ' . $table_name);
		
		$this->db->update(
			$this->fields_table,
			array(
				'display_name' => 0
			), 
			'field_table = "' .  $table_name . '"'
		);
	}
	
	function addFieldToStandardTable($item) {
		$standard_table = $item['field_table'];
		
		// $this->printDebug('AdminDataDefinition::addFieldToStandardTable, standard_table: ' . $standard_table);
		// $this->printDebug($item);
		$columns = $this->db->getColumns($standard_table);
		if (is_array($columns) && in_array($item['field_key'], $columns)) {
			return true;
		}
		// Add standard table field
		$sql = 'ALTER TABLE `' . $standard_table . '` ADD `' . $item['field_key'] . '` '. $this->prepareFieldDef($item);
		
		// $this->db->log_sql = true;
		$this->db->setSqlLogStatus(true, $standard_table);
		
		return $this->db->sqlQuery($sql);
	} // addFieldToCustomTable
	
	function addFieldToCustomTable($item) {
		$custom_table = $this->getCustomTable($item['field_table']);
		// $this->printDebug('AdminDataDefinition::addFieldToCustomTable, custom_table: ' . $custom_table);
		// $this->printDebug($item);
		$columns = $this->db->getColumns($custom_table);
		if (is_array($columns) && in_array($item['field_key'], $columns)) {
			return true;
		}
		// Add custom table field
		$sql = 'ALTER TABLE `' . $custom_table . '` ADD `' . $item['field_key'] . '` '. $this->prepareFieldDef($item);
		
		// $this->db->log_sql = true;
		$this->db->setSqlLogStatus(true, $custom_table);
		
		return $this->db->sqlQuery($sql);
	} // addFieldToCustomTable
	
	function prepareFieldDef($item) {
		// sets up the field definition for an alter table sql statement
		$length = '';
		
		switch ($item['type']) {
			case 'long_text':
				$dtype = 'TEXT';
				bcrm_unset($item, 'default_value'); // Not allowed for this type
			break;
			case 'text':
			case 'password':
			case 'email':
				// case 'hidden':
				$dtype = 'VARCHAR(' . ($item['length'] ? intVal($item['length']) : '255') . ')';
				$item['default_value'] = (is_string($item['default_value']) ? $item['default_value'] : '');
			break;
			case 'hidden': // Phil 20060825, was varchar, now int
			case 'contact': 
			case 'integer':
				$dtype = 'INT(11)';
				$item['default_value'] = intVal($item['default_value']);
			break;
			
			case 'float':
			case 'currency':
			case 'percentage':
				$dtype = 'FLOAT';
				if (function_exists('floatVal')) {
					$item['default_value'] = floatVal($item['default_value']);
				}
			break; 
			
			case 'date':
				$dtype = 'DATE'; // or INT(11) ??
			break;
			case 'integer_date':
				$dtype = 'INT(11)';
			break;
			case 'time':
				$dtype = 'TIME';
			break;
			case 'integer_time':
				$dtype = 'INT(11)';
			break; 
			case 'date_time':
				// $dtype = 'DATETIME';
				// if ($item['_special_date_time']) { // eg created field
					// $dtype = 'INT(11)'; // Phil 20060825 changed to INT(11)
				// }
				// Phil 20070614 All date_time fields treated as timestamps
				$dtype = 'INT(11)';
			break;
			case 'multi_select':
				$dtype = 'VARCHAR(' . ($item['length'] ? intVal($item['length']) : '255') . ')';
			break;
			case 'dropdown':
				// $dtype = 'VARCHAR(255)';
				$dtype = 'INT(11)';
			break;
			case 'checkbox':
				$dtype = 'TINYINT(1)';
			break; 
			case 'file_upload':
				$dtype = 'VARCHAR(255)';
			break;
			case 'free':
				$dtype = 'VARCHAR(' . ($item['length'] ? intVal($item['length']) : '255') . ')';
			break;
			default:
				$dtype = 'VARCHAR(' . ($item['length'] ? intVal($item['length']) : '255') . ')';
			break;
		}
		
		if ($item['non_empty']) {
			$mand = 'NULL';
		} else {
			$mand = 'NOT NULL';
		}
		
		if ($item['default_value']) {
			$default_val = 'default "' . $item['default_value'] . '"';
		} else {
			$default_val = '';
		}
		
		// $this->printDebug('AdminDataDefinition::prepareFieldDef, returning: '  .$dtype . ' ' . $mand . ' ' . $default_val);
		return $dtype . ' ' . $mand . ' ' . $default_val; // eg VARCHAR(255) NOT NULL default "foo"
	} // prepareFieldDef
	
	function removeItem($item) {
		// $this->printDebug('AdminDataDefinition::removeItem');
		// $this->printDebug($item);
		$this->setTable($item['field_table']);
		
		// NOTE: TODO need to change all queries so that if a query uses the
		// deleted field, it is removed from the field list 
		$sql = 'ALTER TABLE `' . $this->getCustomTable($item['field_table']) . '` DROP `' . $item['field_key'] . '`';
		
		// $this->db->log_sql = true;
		$this->db->setSqlLogStatus(true, $item['field_table']);
		
		if ($this->db->sqlQuery($sql)) {
			// Log this
			$this->setLog('Dropped custom table field', $item['field_table'] . '.' . $item['field_key'] . ', id: ' . $item['field_id']);
		} else {	
			// Log this
			$this->setLog('Could not drop custom table field', $item['field_table'] . '.' . $item['field_key']);
		}	
		
		$where_clause = 'field_table = "' . $item['field_table'] . '" AND field_key = "' . $item['field_key'] . '"';		
		
		// $this->db->log_sql = true;
		$this->db->setSqlLogStatus(true, $item['field_table']);
		
		$retval = $this->db->delete(
			$this->fields_table, 
			$where_clause
		);
		
		if ($retval) {
			// Log this
			$this->setLog('Deleted custom field definition', $item['field_table'] . '.' . $item['field_key']);
		} else {	
			// Log this
			$this->setLog('Could not delete custom field definition', $item['field_table'] . '.' . $item['field_key']);
		}
		
		$this->updateDbCache();
		
		return $retval;
	}
	
	// -----------------------------
	
	function getCustomTable($table) {
		if (!empty($table)) {
			$this->setTable($table);
		}
		return $this->table . $this->custom_suffix;
	}
	
	function standardTableExists($table = '') {
		if (!empty($table)) {
			$this->setTable($table);
		}
		if(!isset($this->standard_table_exists)) {
			$this->standard_table_exists = $this->db->tableExists($this->getTable($table));
		}
		return $this->standard_table_exists;
	}
	
	function customTableExists($table = '') {
		if (!empty($table)) {
			$this->setTable($table);
		}

		if (!isset($this->custom_table_exists)) {
			$this->custom_table_exists = $this->db->tableExists($this->getCustomTable($table));
		}
		
		return $this->custom_table_exists;
	}
	
	function createStandardTable($table = '') {
		global $standard_table_structure;
		// $this->printDebug('AdminDataDefinition::createStandardTable table ' . $table);
		
		$standard_table_name = $table;
		
		if (!$this->db->tableExists($table)) {
			$sql = str_replace('__tablename__', $standard_table_name, $standard_table_structure);
			
			// $this->db->log_sql = true;
			$this->db->setSqlLogStatus(true, $standard_table_name);
			
			// if ($retval = $this->db->sqlQuery($sql)) {
			if ($retval = $this->db->createTable($sql)) {
				// Log this
				$this->log->logAction('Created standard table: ' . $standard_table_name);
			} else {	
				// Log this
				$this->log->logAction('Could not create standard table:' . $standard_table_name);
			}
		}
		
		$this->updateDbCache();
		
		return $retval;
	}
	
	function createCustomTable($table = '') {
		global $custom_table_structure;
		// $this->printDebug('AdminDataDefinition::createCustomTable' . $table);
		
		$custom_table_name = $this->getCustomTable($table);
		
		$sql = str_replace('__tablename__', $custom_table_name, $custom_table_structure);
		
		$this->db->log_sql = true;
		$this->db->setSqlLogStatus(true, $custom_table_name);
		
		if ($retval = $this->db->createTable($sql)) {
			// Log this
			$this->log->logAction('Created custom table: ' . $custom_table_name);
		} else {	
			// Log this
			$this->log->logAction('Could not create custom table:' . $custom_table_name);
		}
		
		$this->updateDbCache();
		
		return $retval;
	}
	
	function createDropdownTable($dropdown_table_name, $module_name) {
		global $dropdown_table_structure;
		// $this->printDebug('AdminDataDefinition::createDropdownTable table ' . $dropdown_table_name . ', module ' . $module_name);
		
		// dropdown_table_name must be in this form: ModuleName__tablename_dropdown
		if ( substr($dropdown_table_name, 0, strlen($module_name . '__')) != ($module_name . '__') ) {
			$dropdown_table_name = $module_name . '__' . $dropdown_table_name;
			// $this->printDebug('Prefixed dropdown_table_name:', $dropdown_table_name);
		}
		
		if ( strstr($dropdown_table_name, '_' . $this->dropdown_keyword)  === false ) {
			$dropdown_table_name = $dropdown_table_name . '_' . $this->dropdown_keyword;
			// $this->printDebug('Suffixed dropdown_table_name:', $dropdown_table_name);
		}
		
		$tmp_name = substr($dropdown_table_name, 0, (strlen($dropdown_table_name) - strlen('_' . $this->dropdown_keyword)));
		// print $tmp_name . '<br>';
		// Phil 20051101 Check that the word 'dropdown' only appears once in the table name
		if (strpos($tmp_name, $this->dropdown_keyword) !== false) {
			// print 'error 1';
			return false;
		}
		// Phil 20051101 Check that the word 'custom' is not in the table name
		$custom_keyword = trim($this->custom_suffix, '_');
		if (strpos($tmp_name, $custom_keyword) !== false) {
			// print 'error 2';
			return false;
		}
		
		// Phil 20050329
		$dropdown_table_name = str_replace(' ', '_', $dropdown_table_name);
		
		$sql = str_replace('__tablename__', $dropdown_table_name, $dropdown_table_structure);
		
		$this->db->log_sql = true;
		$this->db->setSqlLogStatus(true, $dropdown_table_name);
		
		// if ($retval = $this->db->sqlQuery($sql)) {
		if ($retval = $this->db->createTable($sql)) {
			
			$this->updateDbCache();
			
			// Log this
			$this->setLog('Created dropdown table', $dropdown_table_name);
			return $dropdown_table_name;
		}
		
		// Log this
		$this->setLog('Could not create dropdown table', $dropdown_table_name);
		return $retval;
	} // createDropdownTable
	
	function createSeenTable($table) {
		global $seen_table_structure;
		// print '<p>seen table for : ' . $table . '</p>';
		// $this->printDebug('AdminDataDefinition::createSeenTable table ' . $table);
		
		$seen_table_name = $table . $this->seen_table_suffix;
		
		if (!$this->db->tableExists($seen_table_name)) {
			$sql = str_replace('__tablename__', $seen_table_name, $seen_table_structure);
			
			// $this->db->log_sql = true;
			$this->db->setSqlLogStatus(true, $seen_table_name);
			// print $sql . '<br>';
			
			if ($retval = $this->db->createTable($sql)) {
				// Log this
				$this->log->logAction('Created item seen table: ' . $seen_table_name);
			} else {	
				// Log this
				$this->log->logAction('Could not create item seen table:' . $seen_table_name);
			}
		}
		
		$this->updateDbCache();
		
		return $retval;
	}
	
	/**
	 * Returns all the tables beginning with the given prefix (usually a module name)
	 */
	function getTables($prefix) {
		$result = $this->db->showTables($prefix);
		$tables = array();
		while ( $row = $this->db->getRow($result) ) {
			if ( count(array_keys($row)) == 1 ) {
				$tables[] = current($row);
			}
		}
		return $tables;
	}
	
	/**
	 * Returns all tables beginning with the given prefix that are not custom tables or dropdown tables
	 */
	function getEditableTables($prefix) {
		$all_tables = $this->getTables($prefix);
		foreach ($all_tables as $k => $v) {
			$suffix = $this->custom_suffix;
			// print $v . ', ends with: ' . substr($v, strlen($v) - strlen($suffix)) . '<br>';
			if (substr($v, strlen($v) - strlen($suffix)) == $suffix) {
				bcrm_unset($all_tables, $k);
			}
		}
		return $all_tables;
	}
	
	/**
	 * Returns all tables beginning with the given prefix that are not custom tables or dropdown tables
	 */
	function getStandardTables($prefix) {
		$all_tables = $this->getTables($prefix);
		// print_r($all_tables); print '<hr>';
		foreach ($all_tables as $k => $v) {
			// print $v . ', ends with: ' . substr($v, strlen($v) - strlen($suffix)) . '<br>';
			if (substr($v, strlen($v) - strlen($this->custom_suffix)) == $this->custom_suffix) {
				bcrm_unset($all_tables, $k);
			} else if (strstr($v, $this->dropdown_keyword) !== false) {
				bcrm_unset($all_tables, $k);
			} else if (substr($v, strlen($v) - strlen($this->seen_table_suffix)) == $this->seen_table_suffix) {
				bcrm_unset($all_tables, $k);
			}
		}
		return $all_tables;
	}
	
	/**
	 * Returns all dropdown tables beginning with the given prefix
	 */
	function getDropdownTables($prefix) {
		$all_tables = $this->getTables($prefix);
		// print_r($all_tables); print '<hr>';
		foreach ($all_tables as $k => $v) {
			if (strstr($v, $this->dropdown_keyword) === false) {
				bcrm_unset($all_tables, $k);
			}
		}
		return $all_tables;
	}
	
	function countFields($table = '') {
		if (!empty($table)) {
			$this->setTable($table);	
		}
		$result = $this->db->quickGet(
			'count(*)', 
			$this->fields_table,
			// 'field_table LIKE "' . $this->getTable() . '"'
			'field_table = "' . $this->getTable() . '"' // Phil 20051004
		);
		return $result;
	}
	
	/**
	 * Returns the given field from the global Data Definition
	 */
	function getField($field_id, $include_empty = false) {
		$item = $this->db->quickGet(
			'*', 
			$this->fields_table,
			"field_id='$field_id'"
		);	
		
		if (!is_array($item)) {
			$item = array();
		}
		
		if (!$include_empty) { // include_empty is a primarily a Module Editor logging call
			// Unsets all '0' or '' values
			foreach($item as $k => $v) {
			// Bosco 20050830 | Added field_is_custom in if condition to include it in item even if empty
				if ( empty($v) && ($k != 'field_is_custom') ) {
					// $this->printDebug('unsetting ' . $k);
					bcrm_unset($item, $k);
				}
			}
		}
		
		return $item;
	}
	
	// --- For editing custom fields ---
	
	function getDateDropdownValues() {
		global $L;
		$dropdown_values = array(
			'0'=>$L['Today'], 
			'1'=> $L['Today'] . ' + 1', 
			'2' => $L['Today'] . ' + 2', 
			'3' => $L['Today'] . ' + 3', 
			'4' => $L['Today'] . ' + 4', 
			'5' => $L['Today'] . ' + 5', 
			'6' => $L['Today'] . ' + 6', 
			'7' => $L['Today'] . ' + 7', 
			'8' =>$L['Today'] . ' + 14', 
			'9' => $L['Today'] . ' + 21', 
			'10' => $L['Today'] . ' + 28', 
			'11' => $L['Today'] . ' + 90'
		);
		return $dropdown_values;
	} // getDateDropdownValues
	
	function getAppendOnlyDropdownValues() {
		global $L;
		return array(
			'0' => $L['No'],
			'1' => $L['Yes'] . ': ' . $L['Space'], 
			'2' => $L['Yes'] . ': ' . $L['DashedLine'], 
			'3' => $L['Yes'] . ': ' . $L['Username'], 
			'4' => $L['Yes'] . ': ' . $L['UsernameAndDatetime']
		);	
	}
	
	function getBooleanDropdownValues() {
		global $L;
		return array(
			'0'=>$L['No'],
			'1'=>$L['Yes'] 
		);
	}	
	
	function getFieldTypeDropdownValues() {
		global $L;
		$dropdown_values = array(
			'text'=>$L['adminCustomFieldsTypeText'], 
			'long_text'=> $L['adminCustomFieldsTypeLongText'], 
			'password' => $L['adminCustomFieldsTypePassword'], 
			'contact' => $L['adminCustomFieldsTypeContact'], 
			'email' => $L['adminCustomFieldsTypeEmail'], 
			'url' => $L['adminCustomFieldsTypeURL'], 
			'hidden' => $L['adminCustomFieldsTypeHidden'], 
			'integer' => $L['adminCustomFieldsTypeInteger'], 
			'float' => $L['adminCustomFieldsTypeFloat'], 
			'currency' => $L['adminCustomFieldsTypeCurrency'], 
			'percentage' => $L['adminCustomFieldsTypePercentage'], 
			'date' => $L['adminCustomFieldsTypeDate'], 
			
			// 'integer_date' => $L['adminCustomFieldsTypeIntegerDate'],  // Removed - not correctly offset for some timezones
			// 'time' => $L['adminCustomFieldsTypeTime'], 
			// 'integer_time' => $L['adminCustomFieldsTypeIntegerTime'], 
			
			'date_time' => $L['adminCustomFieldsTypeDateTime'], 
			
			'multi_select' => $L['adminCustomFieldsTypeMultiSelect'], 
			'dropdown' => $L['adminCustomFieldsTypeDropdown'], 
			
			// 'file_upload' => $L['adminCustomFieldsTypeFileUpload'], // Needs work
			// 'free' => $L['adminCustomFieldsTypeFree'], 
			
			'checkbox' => $L['adminCustomFieldsTypeCheckbox'], 
			'foreign' => $L['adminCustomFieldsTypeForeign'] 
		);
		return $dropdown_values;
	} // getFieldTypeDropdownValues
	
	function getSystemDropdowns() {
		if (is_array($this->system_dropdowns)) {
			return $this->system_dropdowns;	
		}
		return array();
	} // getCustomTableIdField
	
	/**
	 * Return an array of available foreign fields for a given table
	 * @param string $module The current module
	 * @param string $table The current table
	 * @return array $foreign_fields The available foreign fields for the given table. Defaults to an empty array.
	 */
	function getForeignFields($current_module, $current_table) {
		$current_table = $current_module . '__' . str_replace($current_module . '__', '', $current_table);
		global $SERVICES;
		$config =& $SERVICES->getService('Config');
		$items = $config->lookup('modules.Contact.primary_record');
		if (!is_array($items)) {
			$items = array();	
		}
		
		$contact_items = array(
			'Contact' => array('item'), 
			'Company' => array('item'),  
			'Person' => array('item'), 
			'Accounts' => array('item')
		);
		foreach ($contact_items as $k => $v) {
			if (!array_key_exists($k, $items)) {
				$items[$k] = $v;
			}
		}
		
		$fields = array();
		
		$current_fields = $this->getFields($current_table); //TODO: only get foreign fields from this array
		if (!is_array($current_fields)) {
			$current_fields = array();	
		}
		$current_foreign_fields = array();
		foreach ($current_fields as $k => $v) {
			// print_r($v); print '<hr>';
			if (isset($v['foreign']) && $v['foreign']) {
				$current_foreign_fields[] = $v['field_key'];	
			}
		}
		
		// print_r($current_fields); print_r($current_foreign_fields); exit('--');
		// print '<div>current_module: ' . $current_module . '</div>';
		// print '<div>current_table: ' . $current_table . '</div>';
		
		if ($items && is_array($items)) {
			// print '<p>items</p>'; print_r($items);
			foreach ($items as $module => $tables) {
				// if ($module != $current_module) {
					// print '<div>module: ' . $module . '</div>';
					if (is_array($tables)) {
						foreach ($tables as $k => $table) {
							if (($module . '__' . $table) != $current_table) {
								// print '<div>' . $module . '__' . $table . '</div>';
								$all_fields = $this->getFields($module . '__' . $table);
								if (is_array($all_fields)) {
									foreach ($all_fields as $a_k => $a_v) {
										$tmp_field_key = $a_v['field_table'] . '.' . $a_v['field_key'];
										if ( isset($a_v['field_is_custom']) && ($a_v['field_is_custom']==1) ) { // Phil 20051031
											$tmp_field_key = str_replace('_custom', '', $a_v['field_table']) . '_custom.' . $a_v['field_key'];
										}
										
										if (
											(!in_array($tmp_field_key, $current_foreign_fields)) 
											&& ($a_v['field_key'] != 'foreign') 
											&& ($a_v['foreign'] != 1) 
											&& ($a_v['hidden'] != 1) 
											// && ($a_v['dropdown_values'] != '__GROUPS__') 
											// && ($a_v['dropdown_values'] != '__USERS__') 
											&& (!in_array($a_v['dropdown_values'], $this->system_dropdowns)) // Phil 20060814
											&& ($a_v['field_key'] != 'primary_record') 
											&& ($a_v['field_key'] != 'priority') 
											&& ($a_v['field_key'] != 'contact_id') 
											
										) {
											// $fields[] = $a_v['field_table'] . '.' . $a_v['field_key'];
											$fields[] = array (
												'type' => $a_v['type'], 
												'field_key' =>$tmp_field_key, 
												'dropdown_values' => $a_v['dropdown_values'], 
												'label' => $a_v['label'], 
												// 'field_is_custom' => $a_v['field_is_custom'] // Phil 20051031
											);
											
										}
									}
								}
							}
						}
					}
				// }
			}
		}
		// print_r($fields);
		return $fields;
	} // getForeignFields
	
	// --- Dropdown tables ---
	
	/**
	 * Converts System and WebMAT dropdown table names
	 */
	function getDropdownTableName($dropdown_table_name) {
		// print '<p>--- getDropdownTableName --- </p>';
		// For 'WebMAT' dropdown tables ie: __table(Module.table)__
		if (strstr($dropdown_table_name, '__table(')) {
			
			$temp =  explode('.', str_replace(array('__table(', ')__'), '', $dropdown_table_name) );
			$temp =  str_replace(array('__table(', ')__'), '', $dropdown_table_name);
			$temp =  str_replace('.', '__', $temp); 
			// print '<p>--- ' . $temp . ' --- </p>';	
			return $temp;
		}
		
		if (in_array($dropdown_table_name, $this->getSystemDropdowns())) {
			return 	str_replace('__', '', $dropdown_table_name);
		}
		
		return $dropdown_table_name; // No change
	}
	
	function setDropdownTableName($name) {
		// Already a system table
		if (in_array($name, $this->system_dropdowns)) {
			return $name;
		}
		
		$prefix = '__table(';
		$suffix = ')__';
		
		// Convert to 'WebMAT' table name format, eg __table(Module.table)__
		if (!strstr($name, $prefix)) {
			$name = $prefix . trim(str_replace('__', '.', $name)) . $suffix;
		}
		
		return $name;
	}
		
	// functions for manipulating the dropdown field options. 
	
	function getDropdownValues($dropdown_table_name = '') {
		// $this->printDebug('AdminDataDefinition::getDropdownValues()');
		if (empty($dropdown_table_name)) {
			$dropdown_table_name = $this->getTable();
		}
		
		if (in_array($dropdown_table_name, $this->getSystemDropdowns())) {
			// $this->printDebug('returning ' . $dropdown_table_name);
			return 	$dropdown_table_name;
		}
		
		// $this->printDebug('dropdown table name: ' . $dropdown_table_name . '');
		$dropdown_table_name = $this->getDropdownTableName($dropdown_table_name);
		// $this->printDebug('dropdown table name: ' . $dropdown_table_name . '');
		
		$result = $this->db->select(
			'*', 
			$dropdown_table_name, 
			'active = 1', 
			'priority, name'
		);
		
		if ($this->db->hasError())	{
			$this->db->printError();
		}
		
		$dropdowns = array();
		
		while ($row = $this->db->getRow($result)) {
			// print '<p>name: ' . $row['name'] . '</p>'; exit('---');
			$row['name'] = $this->processLanguageText($row['name']);
			$dropdowns[$row['id']] = $row['name'];
		}
		return $dropdowns;
		
	} // getDropdownValues
	
	function getDefaultDropdownValue($field_key, $dropdown_table_name = '') {
		// $this->printDebug('AdminDataDefinition::getDefaultDropdownValue(' . $field_key . ')');
		if (empty($dropdown_table_name)) {
			$dropdown_table_name = $this->getTableName();
		}
		
		// ***** Needs work *****
		// $result = $this->db->quickGet(
			// 'dropdown_values', 
			// $this->fields_table, 
			// "field_key = '$field_key'"
		// );
		$result = $this->db->quickGet(
			'default_value', 
			$this->fields_table, 
			"field_key = '$field_key' AND field_table = '$dropdown_table_name'"
		);
		
		return $result;
	}
	
	function getDropdownValue($id, $dropdown_table_name = '') {
		// $this->printDebug('AdminDataDefinition::getDropdownValue(' . $id . ')');
		if (empty($dropdown_table_name)) {
			$dropdown_table_name = $this->getTable();
		}
		// $this->printDebug('AdminDataDefinition::getDropdownValue, table name: ' . $dropdown_table_name);
		
		$dropdown = $this->db->quickGet(
			'*', 
			$dropdown_table_name, 
			"id='$id'"
		);
		
		if ($this->db->hasError())	{
			$this->db->printError();
		}
		return $dropdown;  
	} // getDropdownValue
	
	function setDropdownValue($dropdown, $dropdown_table_name = '', $check_exists = false) {
		// Phil 20060817 Added param check_exists (default false).
		// When not false, check whether $dropdown[$check_exists] is already a value in the db
		
		// print_r($dropdown); print '<hr>'; exit();
		global $USER;
		// $this->printDebug('AdminDataDefinition::setDropdownValue()');
		// $this->printDebug($dropdown);
		
		if (empty($dropdown_table_name)) {
			$dropdown_table_name = $this->getTableName();
		}
		
		if ($dropdown['id']) {
			$id = $dropdown['id'];
			$temp = $this->db->quickGet(
				'*', 
				$dropdown_table_name, 
				"id='$id'"
			);
		} else if ($check_exists !== false) {
			if (isset($dropdown[$check_exists])) {
				$check_field = $check_exists;
				$check_value = $dropdown[$check_exists];
				$temp = $this->db->quickGet(
					'*', 
					$dropdown_table_name, 
					"`" . $check_field . "`='$check_value'"
				);
				if (is_array($temp) && isset($temp['id'])) {
					$id = $temp['id'];	
				}
			}
		}
		
		bcrm_unset($dropdown, 'table');
		bcrm_unset($dropdown, 'id');
		
		// print '<div>add->setDropdownValue temp:</div>'; print_r($temp); print '<hr>';
		if ($temp) {
			$this->db->log_sql = true;
			$this->db->setSqlLogStatus(true, $dropdown_table_name);
			
			$this->db->update(
				$dropdown_table_name, 
				$dropdown, 
				// "id='$id'"
				'`name`="' .  $temp['name'] . '"'
			);
			
			if ($this->db->hasError())	$this->db->printError();
			
			return ($id);
			
		} else {
			// $this->db->log_sql = true;
			$this->db->setSqlLogStatus(true, $dropdown_table_name);
			$this->db->insert($dropdown_table_name, $dropdown); 
			if ($this->db->hasError())	$this->db->printError();
			return ($this->db->getInsertId());
		}
		return false;
	} // setDropdownValue
	
	function deleteDropdownValue($id, $dropdown_table_name = '') {
		// $this->printDebug('AdminDataDefinition::setDropdownValue(' . $id . ')');
		if (empty($dropdown_table_name)) {
			$dropdown_table_name = $this->getTableName();
		}
		
		if ($item = $this->getDropdownValue($id, $dropdown_table_name)) {
			// $this->db->log_sql = true;
			$this->db->setSqlLogStatus(true, $dropdown_table_name);
			
			$this->db->update(
				$dropdown_table_name, 
				array("active" => "0"), 
				// "id='$id'"
				'`name`="' .  $item['name'] . '"'
			); 
			
			if ($this->db->hasError())	{
				$this->db->printError();
			} else {
				$this->setLog('Deleted dropdown value', $dropdown_table_name . ' id = ' . $id);
			}	
		}
	} // deleteDropdownValue
	
	function updateDbCache() {
		$this->db->rebuildCache();
	}
	
	function setLog($title, $msg) {
		global $SERVICES;
		$log =& $SERVICES->getService('Log');
		$log->logAction($title . ': ' . $msg);
	}
}

/**
 * New Database Table Structures
 * @access private
 */
global $standard_table_structure, $custom_table_structure, $seen_table_structure, $dropdown_table_structure, $dropdown_table_structure;

/**
 * @access private
 */
$custom_table_structure = '
CREATE TABLE `__tablename__` (
	`custom_id` int(11) NOT NULL auto_increment,
	PRIMARY KEY  (`custom_id`)
) TYPE=MyISAM
';

/**
 * @access private
 */
$standard_table_structure = '
CREATE TABLE `__tablename__` (
	`id` int(11) NOT NULL auto_increment,
	PRIMARY KEY  (`id`)
) TYPE=MyISAM
';

/**
 * @access private
 */
$seen_table_structure = "
CREATE TABLE `__tablename__` (
  `item_id` int(11) NOT NULL default '0',
  `user_id` int(11) NOT NULL default '0',
  `seen` tinyint(1) NOT NULL default '0',
  PRIMARY KEY  (`item_id`,`user_id`)
) TYPE=MyISAM
";

/**
 * @access private
 */
$dropdown_table_structure = "
CREATE TABLE `__tablename__` (
  `id` int(11) NOT NULL auto_increment,
  `name` varchar(64) NOT NULL default '',
  `priority` int(3) NOT NULL default '100',
  `active` tinyint(1) NOT NULL default '1',
  PRIMARY KEY  (`id`),
  KEY `priority` (`priority`),
  KEY `active` (`active`)
) TYPE=MyISAM 
";

?>