<?php
/**
 *
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Services
 */
 
includeService('Authentication');

/**
 * Manages 'Group' functionality for the Authentication Service.
 *
 * @author Phil Calder
 * @package Services
 */
class Groups extends Service {
	/**
	 * @access private
	 */
	var $db = '';
	/**
	 * @access private
	 */
	var $groups_for = '';
	/**
	 * @access private
	 */
	var $groups_list = '';
	/**
	 * @access private
	 */
	var $administrators_gid = false;
	/**
	 * @access private
	 */
	var $everyone_gid = false;
	
	
	/**
	 * Groups constructor
	 *
	 * The Authentication service calls this constructor
	 * @param object $db - a database object (see the Database service)
	 */
	function Groups() {
		global $SERVICES;
		$this->db =& $SERVICES->getService('Database'); // Must already exist since we dont know the args	
		
		$this->groups_for = array();
		$this->groups_list = array();
	}
	
	/**
	 * Returns an array of all the groups visible to the current user
	 *
	 * A group if visible to a user if that user is in the group, 
	 * or the group itself is visible (ie not a private group).
	 *
	 * @param $type Optional, will return all types of groups (privileges, teams, private)
	 * @param $all Optional, will return all groups (including private) if the current user has user admin 'admin' privileges.
	 * @param $limit Optional, for paging.
	 * @param $all_fields Optional, will return all group fields (default is name).
	 * @return array of groups visible to the current user (key is the group id, value is the group name, unless $all_fields parameter is true)
	 */
	function getGroupList($type = 'all', $all = false, $limit = '', $all_fields = false) {
		global $USER;
		
		if (empty($type)) {
			$type = 'all';
		}
		
		$user_id = $USER['user_id'];
		
		$visibility = 'visible=1';
		
		if ($all) {
			
			global $SERVICES;
			
			$ua =& $SERVICES->getService('Authentication');
			if ($ua->adminAccess($user_id, 'UserAdmin')) {
				
				$visibility = '1'; // ie no visible=1 in where clause
				
			}
			
		}
		
		if ($all_fields) {
			$fields = '*';
		} else {
			$fields = 'id, name';
		}
		
		$group_ids  = $this->getGroupsForUser($user_id);
		$group_ids = implode(',', array_keys($group_ids));
		
		$where = '(id IN (' . $group_ids . ') OR ' . $visibility . ')';
		
		if ($type != 'all') {
			switch ($type) {
				case 'team':
					$type_id = 2;
				break;
				case 'privs':
					$type_id = 0;
				break;
				case 'private':
					$type_id = 1;
				break;
			}
			
			$where .= ' AND (type = ' . $type_id . ')';
		}
		
		$result = $this->db->select(
			$fields, 
			'System__groups', 
			$where, 
			'type DESC, name ASC', // Phil 20050714 Chacnge to type DESC, order will be: teams, private, then permissions groups
			'',
			$limit
		);
		
		$groups = array();
		
		while ($row = $this->db->getRow($result)) {
			
			// $this->groups_list[$user_id][$row['id']] = $row['name'];
			if (!$all_fields) {
				$groups[$row['id']] = $row['name'];
			} else {
				$groups[$row['id']] = $row;
			}
			
		}
		
		// return $this->groups_list[$user_id]; // wasnt working with paging (ie limit)
		return $groups;
		
	}
	
	/**
	 * Deprecated
	 * @see getGroupList
	 */
	function getGroupListInclusive($limit = '') {
		
		return $this->getGroupList('all', true, $limit);
		
	}

	/**
	 * @return array of all groups, key is group id, value is group name.
	 * @see getGroupsList
	 * @see getGroupsForUser
	 */
	function getGroups() {
		
		$result = $this->db->select(
			'System__groups.id, System__groups.name', 
			'System__groups', 
			'name'
		);
		
		$items = array();
		
		while ($row = $this->db->getRow($result)) {
			
			$items[$row['id']] = $row['name'];
			
		}
		
		return $items;
		
	}	
	
	
	/**
	 * 
	 *
	 * @param string $user_id The user id to get the groups for
	 * @return array of groups the user belongs to, key is group id, value is group name.
	 */
	function getGroupsForUser($user_id) {
		
		if (isset($this->groups_for[$user_id]) && is_array($this->groups_for[$user_id])) {
			
			return $this->groups_for[$user_id];
			
		}
		
		$result = $this->db->select(
			'System__groups.id, System__groups.name', 
			'System__groups, System__user_group', 
			'System__user_group.group_id = System__groups.id AND System__user_group.user_id = ' . $user_id,
			'System__groups.name'
		);
		
		$this->groups_for[$user_id] = array();
		
		while ($row = $this->db->getRow($result)) {
			
			$this->groups_for[$user_id][$row['id']] = $row['name'];
			
		}
		
		return $this->groups_for[$user_id];
		
	}
	
	/**
	 * @param integer $id The id of the group to look for.
	 * @return array containing group information (keys: name, visible, editable), or false.
	 */
	function getGroup($id) {
		
		if (empty($id)) {
		
			return array();
			
		}
		
		return $this->db->quickGet(
			'*',
			'System__groups',
			'id = ' . $id
		);
		
	}
	
	/**
	 * Deprecated
	 *
	 * @see getGroup
	 */
	function getGroupDetails($id) {
		
		return $this->getGroup($id);
		
	}
	
	
	/** 
	 * Gets the users in a group or groups
	 *
	 * @param integer $id The id of the group - OR array $id An array of group ids
	 * @return array of users in this group (key is user id, value is username)
	 */
	function getUsers($id) {
		
		if (empty($id)) {
			
			return array();
			
		}
		
		if (is_array($id) && sizeof($id)) {
			$ids_str = implode(', ', $id);
			$where = "System__users.id = System__user_group.user_id AND System__user_group.group_id IN($ids_str)";
		} else {
			$where = "System__users.id = System__user_group.user_id AND System__user_group.group_id='$id'";
		}
		
		$result = $this->db->select(
			'System__users.id, System__users.username', 
			'System__users, System__user_group', 
			$where, 
			'System__users.username'
		);
		
		$items = array();
		
		while ($row = $this->db->getRow($result)) {
			
			$items[$row['id']] = $row['username'];
			
		}
		
		return $items;
		
	}
	
	/**
	 * @param integer $id The group id
	 * @return boolean false, or 1.
	 */
	function isEditable($id) {
		
		if (empty($id)) {
		
			return false;
			
		}
		
		return $this->db->quickGet(
			'editable',
			'System__groups',
			'id = ' . $id
		);
		
	}
	
	/**
	 * @param string $name The name of the group.
	 * @return integer id, the group id, or false.
	 */
	function getGroupId($name) {
		
		$id = $this->db->quickGet(
			'id', 
			'System__groups',
			'name = "' . $name . '"'
		);
		
		return $id;
	}
	
	/**
	 * @param integer $id The id of the group.
	 * @return string name, the group name, or false.
	 */
	function getGroupName($id) {
		
		$id = $this->db->quickGet(
			'name', 
			'System__groups',
			'id = ' . $id
		);
		
		return $id;
	}
	
	
	/**
	 * @param integer $id the id of the user
	 * @return integer $id of the user's private group.
	 * @author Matthew Ross
	 */
	function getPrivateGroupForUser($id) {
		
		$id = $this->db->quickGet(
			'private_group_id', 
			'System__users',
			'id = ' . $id
		);
		
		return $id;
	}
	
	
	/**
	 * Only available to user administrators.
	 *
	 * Use of this function is logged. Removes all users from this group, then removes the group.
	 * @param integer $id The id of the group.
	 */
	function removeGroup($id) {
		global $USER, $SERVICES;
		
		$log =& $SERVICES->getService('Log');
		
		$ua =& $SERVICES->getService('Authentication');
		
		if (!empty($id) && $ua->adminAccess($USER['user_id'], 'Authentication')) {
			
			if ($this->isEditable($id)) {
				
				$this->db->delete(
					'System__user_group', 
					"group_id = '$id'"
				);
				
				$this->db->delete(
					'System__groups', 
					"id = '$id'"
				);
				
				$log->logAction('Removed group ' . $id);
				
			}
			
		}
		
	}
	
	/**
	 * Adds or removes a user from a specified group.
	 *
	 * @param integer $user_id The id of the user to add to/remove from the group.
	 * @param integer $group_id The group to add or remove the user from.
	 * @param integer $status Evaluates to boolean, 0 - remove user from group, 1 - add user to group. 
	 */
	function setUser($user_id, $group_id, $status) {
		global $SERVICES;
		
		$log =& $SERVICES->getService('Log');
		
		if (
			empty($status) && 
			!( ($group_id == $this->getAdministratorsGroupId())  && ($user_id == $this->getAdminUserId()) ) 
		) { // Phil 20050309 - Cannot remove Admin user from Administrators group
			
			$this->db->delete(
				'System__user_group', 
				'user_id=' . $user_id . ' AND group_id=' . $group_id
			);
			
			$log->logAction('User ' . $user_id . ' removed from group ' . $group_id);
			
		} else {
		
			// Phil - Refreshing a page after adding a user caused a duplicate entry error without this check
			if ($group_id && $user_id && !$this->db->quickGet('count(*)', 'System__user_group', 'user_id=' . $user_id . ' AND group_id=' . $group_id )) { 
				
				$this->db->insert( 
					'System__user_group', 
					array('user_id'=>$user_id,'group_id'=>$group_id) 
				);
				
				$log->logAction('User ' . $user_id . ' added to group ' . $group_id);
				
			}
			
			if ($this->db->hasError()) {
				
				$this->db->printError();
				
			}
		
		}
		
	}
	
	/** 
	 * Add a new group. Prints an error if unable to add group.
	 *
	 * @param array $item, containig information about the new group. Keys: name (string), visible (1 or 0), editable (1 or 0).
	 * @return false, or the new group id
	 */
	function addGroup($item) {
		
		if (isset($item['name']) && $item['name']) {
			if ($existing_group = $this->getGroupId($item['name'])) {
				return false;
			}
		}
		
		$id = $this->db->insert(
			'System__groups', 
			$item
		);
		
		if ($this->db->hasError()) {
			
			$this->db->printError();
			
		} else {
			
			global $SERVICES;
			$log =& $SERVICES->getService('Log');
			
			$log->logAction('Added group ' . $id);
			
		}
		
		return $id;
		
	}
	
	/** 
	 * Updates the given group
	 *
	 * @param integer $id, the id of the group to update.
	 * @param array $item, containing information about the new group. Keys: name (string), visible (1 or 0), editable (1 or 0).
	 * @return boolean true or false
	 */
	function setGroup($id, $item) {
		
		if ($id && $this->isEditable($id)) {
			
			if (!isset($item['visible']) || !$item['visible']) {
				
				$item['visible'] = 0;
				
			}
			
			$this->db->update(
				'System__groups', 
				$item, 
				'id=' . $id
			);
			
			if ($this->db->hasError()) {
				
				$this->db->printError();
				
				return false;
				
			}
			
			return true;
			
		}
		
		return false;
		
	}
	
	/**
	 * Update the privilege level for a group to a module
	 *
	 * @param integer $group_id, the id of the group.
	 * @param integer $module_id, the id of the module.
	 * @param integer $level, the privilege level to set (1 = read, 2 = write, 4 = admin or any combination up to 7 - read, write, admin).
	 */
	function setPrivileges($group_id, $module_id, $level) {
		
		if ($this->getPrivileges($group_id, $module_id)) {
			
			$this->db->update(
				'System__module_group', 
				array('priv' => $level), 
				'module_id = ' . $module_id . ' and group_id = ' . $group_id
			);
		
		} else {
			
			$this->db->insert(
				'System__module_group',
				array(
					'priv' => $level, 
					'module_id' => $module_id, 
					'group_id' => $group_id
				)
			);
			
		}
		
	}
	
	/**
	 * Get the privilege level for a group to a module
	 *
	 * @param integer $group_id, the id of the group.
	 * @param integer $module_id, the id of the module.
	 * @return integer $level, the privilege level (1 = read, 2 = write, 4 = admin or any combination up to 7 - read, write, admin).
	 */
	function getPrivileges($group_id, $module_id) {
		
		return $this->db->quickGet(
			// 'group_id, module_id, priv',
			'*', 
			'System__module_group',
			'module_id = ' . $module_id . ' and group_id = ' . $group_id
		);
	}
	
	function getAdministratorsGroupId() {
		if ($this->administrators_gid) {
			return $this->administrators_gid;
		}
		
		$result = $this->db->select(
			'id', 
			'System__groups', 
			'name=\'Administrators\' AND editable=\'0\''
		);
		if($this->db->hasError()) {
			$this->db->printError();
		}
		while ($row = $this->db->getRow($result)) {
			$this->administrators_gid = $row['id'];
			break;
		}
		
		return $this->administrators_gid;
	}
	
	function getEveryoneGroupId() {
		if ($this->everyone_gid) {
			return $this->everyone_gid;
		}
		$result = $this->db->select(
			'id', 
			'System__groups', 
			'name=\'Everyone\' AND editable=\'0\''
		);
		if($this->db->hasError()) {
			$this->db->printError();
		}
		while ($row = $this->db->getRow($result)) {
			$this->everyone_gid = $row['id'];
			break;
		}
		return $this->everyone_gid;
	}
	
	function getAdminUserId() {
		global $SERVICES;
		if ($users =& $SERVICES->getService('Users')) {
			return $users->getAdminUserId();
		}
		return 1; //default
	}
	
}
?>