<?php
/**
 * Modules.
 *
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Services
 */

/**
 * Modules are returned by reference (ie singleton instance). 
 * 
 * Use '=&' when calling {@link getModule()}
 * 
 * @author Phil Calder
 * @package Services
 */
class Modules extends Service {
	 
	/**
	 * @access private
	 */
	var $current = false;
	/**
	 * @access private
	 */
	var $modules = '';
	/**
	 * @access private
	 */
	var $list = '';
	/**
	 * @access private
	 */
	var $mod_error = false;

	/**
	 * Get a specified module (or the current module) object.
	 * 
	 * The getModule function returns by reference (ie a singleton instance of the module).
	 *
	 * @param string $module Optional module name. Will attempt to determine the current module if none provided.
	 * @return mixed The specified (or default) module object, or false if there was a problem loading or initialising the object.
	 */
	function &getModule($module = '') {
		
		if (empty($module)) {
			// $this->printDebug('Modules::getModule(' . $module . ') - getting current module');
			$module = $this->getCurrent();
		}
		
		if ($this->exists($module)) {
			// $this->printDebug('Modules::getModule(' . $module . '), exists');
			if (isset($this->modules) && is_array($this->modules) && isset($this->modules[$module]) && class_exists($module)) {
				// $this->printDebug('Modules::getModule(' . $module . '), already cached');
				return $this->modules[$module];
			}
			// $this->printDebug('Modules::getModule(' . $module . '), getting Config service');
			global $SERVICES;
			$SERVICES->getService('Config');
			// $this->printDebug('Modules::getModule(' . $module . '), got Config service');
			if (! ($class_file = $SERVICES->config->lookup('modules.' . $module . '.classfile'))) {
				$class_file = 'inc/classes.inc'; // default
			}
			// $this->printDebug('Modules::getModule(' . $module . '), including class file');
			if (!includeModule($module, $class_file)) {
				// $this->printDebug('Modules::getModule(' . $module . '), returning false (includeModule returned false)');
				$this->mod_error = false; // need to return an object variable
				return $this->mod_error;
			}
			// $this->printDebug('Modules::getModule(' . $module . '), included class file');
			// $this->printDebug('Modules::getModule(' . $module . '), getting Language service service');
			// Language file (?)
			$SERVICES->getService('Language');
			$SERVICES->language->getLanguage($module);
			// $this->printDebug('Modules::getModule(' . $module . '), got Language service');
			// $this->printDebug('Modules->getModule(' . $module . ') about to create object');
			if (class_exists($module)) {
				// $this->printDebug('Modules->getModule(' . $module . ') instanciating new object');
				$this->modules[$module] = new $module;
				// $this->printDebug('Modules->getModule(' . $module . ') instanciated new object');
			} // else {
				// $this->printDebug('Modules->getModule(' . $module . '): ' . $module . ' class does not exists');
			// }
			// $this->printDebug('Modules::getModule(' . $module . '), returning object');
			return $this->modules[$module];
		}
		// $this->printDebug('getModule: ' . $module . ', returning false');
		$this->mod_error = false; // need to return an object variable
		return $this->mod_error;
	}
	
	/**
	 * Queries the system database for a list of available modules.
	 *
	 * @param int $type Optional, defaults to 2 (all). Types are 0: standard, 1: custom, 2: all (standard and custom)
	 * @return array An array of modules (index is the module id, value is the module name)
	 */
	function getModules($type = 2, $refresh = false) {
		if ($refresh || !isset($this->module_list) || !is_array($this->module_list)) {
			global $SERVICES;
			$SERVICES->getService('Database');
			$result = $SERVICES->database->select(
				'id, name, is_custom', 
				'System__modules', 
				'', 
				'name'
			);
			
			$this->module_list = array();
			$this->custom_module_list = array();
			$this->standard_module_list = array();
			
			while ($row = $SERVICES->database->getRow($result)) {
				$this->module_list[$row['id']] = $row['name'];
				if ($row['is_custom']) {// Phil 20051021
					$this->custom_module_list[$row['id']] = $row['name'];
				} else {
					$this->standard_module_list[$row['id']] = $row['name'];
				}
			}
			// $this->printDebug('Modules, getModules:');
			// $this->printDebug($this->module_list);
		}
		
		switch ($type) { // Phil 20051021
			case 0:
			return (array)$this->standard_module_list;
			break;
			case 1:
			return (array)$this->custom_module_list;
			break;
			default:
			return (array)$this->module_list;
			break;
		}
		return $this->module_list;
	}
	
	/**
	 *
	 * @return array An array of Standard modules (index is the module id, value is the module name)
	 */
	function getStandardModules() {
		// Phil - Modified 20051021
		return $this->getModules(0); // custom
	}
	
	/**
	 *
	 * @return array An array of Custom modules (index is the module id, value is the module name)
	 */
	function getCustomModules() {
		// Phil - Modified 20051021
		return $this->getModules(1); // custom
	}
	
	/**
	 * @param mixed $identifier - Can be either a module id or name
	 * @return mixed 0=standard module, 1=custom module, false=unknown 
	 * (or module does not exist, see Modules::exists()
	 */
	function isCustom($identifier) {
		// Phil - Modified 20051021
		$custom_modules = $this->getModules(1); // custom
		// TODO - alter getModules, check here whether we have cached this result (this->module_list)
		if (is_numeric($identifier)) {
			// $where = "id='$identifier'";
			return array_key_exists($identifier, $custom_modules);
		} else {
			// $where = "name='$identifier'";
			return in_array($identifier, $custom_modules);
		}
	}
	
	/**
	 * Queries the system database for a list of 'default' modules.
	 *
	 * @return array An array of modules (numeric index, value is the module name)
	 */ 
	function getDefaultModules() {
		global $SERVICES;
		$db =& $SERVICES->getService('Database');
		$result = $db->select(
			'*', 
			'System__default_modules', 
			'for_new_install = 1'
		);
		$items = array();
		while ($row = $db->getRow($result)) {
			$items[] = $row;
		}
		return $items;
	}
	
	/**
	 * Attempts to determine the name of the current module.
	 *
	 * edit by Brenton 20050317 to better determine names by first
	 * using the script_filename
	 *
	 * Note: There may not be a 'current module'.
	 * @return string The current module name or a blank string
	 */
	function getCurrent() {
		// print '<div>Modules::getCurrent()</div>';
		if (isset($this->current) && ($this->current !== false)) {
			// print '<p>cached current: ' . $this->current . '</p>';
			return $this->current;
		}
		// print '<div>calculating current</div><hr>';
		global $SERVICES, $BCRM_DOCUMENT_ROOT;
		$modules_dir = $this->getModulesBaseDir(); // $SERVICES->config->lookup('modules.basedir');
		$m_len = strlen($modules_dir);
		// TODO: Phil, 20060630, test script_filename on Windows IIS, Apache CGI
		// if there's a script_filename, we will default to use that, 
		// since it's more accurate of what we want to get
		// and typically will also handle symlinks and userdirs
		// use 'realpath' to expand symlinks and stuff
		$script_filename = realpath($SERVICES->getGlobal('script_filename'));
		if (strncmp( $modules_dir, $script_filename, $m_len ) == 0) {
			$path = $script_filename;
		} else {
			$subdir = $this->getWebSubDir(); // $SERVICES->config->lookup('site.websubdir');
			if ($subdir) {
				$file_path = substr($BCRM_DOCUMENT_ROOT, 0, (strlen($BCRM_DOCUMENT_ROOT) - strlen($subdir)));
			} else {
				$file_path = $BCRM_DOCUMENT_ROOT;
			}
			$file_path = $file_path . $SERVICES->getGlobal('php_self');
			$file_path = str_replace('//', '/', $file_path);
			if (strncmp( $modules_dir, $file_path, $m_len ) == 0) {
				$path = $file_path;
			}
		}
		// if we actually have a value (as determined above), then set it
		if (isset($path) && strlen($path) > 0) {
			$path = str_replace( $modules_dir, '', $path );
			$path = trim($path, '/');
			$path = explode('/', $path);
			$this->setCurrent($path[0]);
		} else {
			$this->setCurrent(); 
		}
		// $this->cache['current_module'] = $this->getCurrent();
		// $this->printDebug('Modules, getCurrent returning ' . $this->current);
		return $this->current;
	}
	
	/**
	 * Sets the current module
	 *
	 * @param $str The name of the module
	 */
	function setCurrent($str = '') {
		 $this->current = $str;
	}
	
	/**
	 * @param $module The name of the requested module.
	 * @return boolean true or false if the given module exists.
	 */
	function exists($module) {
		if (empty($module)) {
			return false;
		}
		return (in_array($module, $this->getModules()));
	}
	
	/**
	 * @access private
	 * @return string The web sub-directory
	 */
	function getWebSubDir() {
		if (!isset($this->web_sub_dir)) {
			global $SERVICES;
			$SERVICES->getService('Config');
			$this->web_sub_dir = $SERVICES->config->lookup('site.websubdir');
		}
		return $this->web_sub_dir;
	}
	
	/**
	 * @access private
	 * @param string $type The module-type (options are 'standard' (default), 'custom', 'extension'
	 * @return string The (specified) modules base directory
	 */
	function getModulesBaseDir($type = 'standard') {
		if (!isset($this->modules_basedir)) {
			global $SERVICES;
			$SERVICES->getService('Config');
			$this->modules_basedir = $SERVICES->config->lookup('modules.basedir');
			$this->modules_custom_basedir = $SERVICES->config->lookup('modules.custom_basedir');
			$this->modules_extension_basedir = $SERVICES->config->lookup('modules.custom_basedir');
		}
		switch ($type) {
			case 'standard':
				return $this->modules_basedir;
			break;
			case 'custom':
				return $this->modules_custom_basedir;
			break;
			case 'extension':
				return $this->modules_extension_basedir;
			break;
		}
		return $this->modules_basedir;
	}
	
	/**
	 * @param string $module The module name
	 * @param boolean $relative Default is false - links are absolute from the site root. When true, links are relative to any module directory (standard or custom)
	 * @return mixed false or web path to module
	 * @author Phil Calder
	 */
	function getModulePath($module, $relative = false) {
		global $BCRM_DOCUMENT_ROOT;
		if ($this->exists($module)) {
			if (!$relative) {
				$web_path = '/' . $this->getWebSubDir();
			} else {
				$web_path = '../..'; // relative to any module directory
			}
			if ($this->isCustom($module)) {
				$path = $web_path . str_replace($BCRM_DOCUMENT_ROOT, '', $this->getModulesBaseDir('custom')) . '/' . $module;
				
			} else {
				
				
				$path =  $web_path . str_replace($BCRM_DOCUMENT_ROOT, '', $this->getModulesBaseDir('standard')) . '/' . $module;
			}
			if (substr($path, 0, 2) == '//') {
				$path = substr($path, 1);
			}
			return $path;
		}
		return false;
	}
}
?>