<?php
/**
 * @license http://www.browsercrm.com/licence.php BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package WebMAT
 */

includeWebMAT('item.inc');
includeWebMAT('data_dictionary.inc');

/**
 * This class handles and displays an item edit page
 *
 * @package WebMAT
 * @author Phil Calder, Matthew Ross, Matthew Corich, Simon Cook, Brenton LeTrabb
 * @abstract
 */
class WebMATStandardEdit extends WebMATItem {
	
	/**
	 * @var integer $delete the id of the item to be deleted
	 * @access private
	 */
	var $delete = false;
	/**
	 * @var boolean $submit used to check if the item has been submitted
	 * @access private
	 */
	var $submit = false;
	/**
	 * @var integer $form_js_validate sets javascript state for the form
	 * 0 is off
	 * 1 is on (means add js call to form tag AND autogenerate field validation)
	 * 2 is partial on; means just add js call to form tag
	 * @access private
	 */
	var $form_js_validate = 1;
	
	/**
	 * @access private
	 */
	var $restrict_level = 'write';
	
	/**
	 * init function sets all the class variables
	 *
	 * @param  string $module  the current module name
	 * @param  string $data_option  the current data option used for DataDictionary
	 * @param  array $post the post array
	 * @param  array $get the get array
	 * @param  string $table_name the name of the table the page is using
	 * @param  array $language_overwrite has the keys title and description
	 */
	function init($module, $data_option, $post = false, $get = false, $table_name = false, $language_overwrite = false) {
		parent::init($module, $data_option, $post, $get, $table_name, $language_overwrite);
		$this->disableReadOnly();
		$this->setSubmitVariable();
		$this->setDeleteVariable();
	}
	
	/** 
	 * @author Phil Calder
	 */
	function substituteTagContents() {
		$this->output->substituteTagContent('outerTable', 'outerTableEdit');
	}

	/**
	 * returns the WebMAT type
	 * @return string WebMAT type
	 */
	function getWebmatType() {
		return 'edit';
	}

	/**
	 * disables the read only fields for a new item
	 */
	function disableReadOnly() {
		if (!$this->form_values['id'] && !$this->form_values['incl_read_only']) {
			$this->dd->setNoReadOnly();
		} else if ($this->form_values['incl_read_only']) {
			$this->dd->setNotReadOnly('__all__');
			$this->db_functions->setNotReadOnly('__all__');
		}
	}

	/**
	 * Removes foreign fields from the item array
	 *
	 * @param array $item The item array
	 * @return array The item with foreign fields removed
	 * @author Phil Calder
	 */
	function removeForeignFields($item) {
		$foreign_fields = $this->dd->getForeignFields();
		if (is_array($foreign_fields)) {
			foreach ($foreign_fields as $k => $v) {
				bcrm_unset($item, $v);
			}
		}
		return $item;
	}

	/**
	 * sets the submit variable
	 */
	function setSubmitVariable() {
		if ($this->form_values['submit']) {
			$this->submit = $this->form_values['submit'];
			$this->delete = false;
		}
	}

	/**
	 * returns the submit variable
	 *
	 * @return boolean true if submit exists
	 */
	function getSubmitVariable() {
		return $this->submit;
	}

	/**
	 * sets the delete variable
	 * @access private
	 */
	function setDeleteVariable() {
		if ($this->form_values['delete']) {
			$this->delete = $this->form_values['delete'];
		}
	}

	/**
	 * returns the delete variable
	 *
	 * @return integer the id of the item to be deleted
	 * @access private
	 */
	function getDeleteVariable() {
		return $this->delete;
	}


	/**
	 * Call prior to displaying the page. This method handles submit or delete functionality
	 * and sets the return page variable.
	 */
	function run() {
		if (isset($this->form_values['submit']) || isset($this->form_values['delete'])) {
			$this->setLogAction();
			if ($this->form_values['delete']) {
				$this->setRefererAsReturnPage();
			}
			
			$this->doSubmit();
		}
		$this->setRefererAsReturnPage();
	}

	/**
	 * set the log actions
	 * @author Syed
	 */
	function setLogAction() {
		if ($this->form_values['submit']) {
			if ($this->form_values['item']['id']) { //Syed 20050831
				$this->db_functions->setLogAction('update');
			} else {
				$this->db_functions->setLogAction('create');
			}
		} else {
				$this->db_functions->setLogAction('delete');
		}
	}
	
	/**
	 * Suffix for the description row language text key
	 *
	 * 
	 * @return string the description row content language key suffix
	 */
	function getDescriptionContentSuffix() {
		return 'EditDescription';
	}
	
	/**
	 * Calls {@link main} and exits. Override for custom validation error handling
	 *
	 * @see doSubmitValidate
	 * @see main
	 * @author Phil Calder
	 */
	function doSubmitValidateErrorHandler($id = '', $item = '', $error_msg = '') {
		$this->main($id, $item, $error_msg);
		exit();
	}

	/**
	 * Builds the edit form
	 *
	 * This method: 
	 * gets the item
	 * checks that it is editable
	 * set the primary record flag
	 * calls {@link beforeForm}
	 * calls {@link handlePermissions}
	 * calls its parent method
	 *
	 * @param  integer  $id  an item id
	 * @param  array  $item  the item array from a form that failed validation
	 * @param  array  $error_msg  an array of error messages.
	 */
	function main($id = '', $item = '', $error_msg = '') {
		global $SERVICES, $L;
		// print "<div>" . get_class($this) . " - wme - main($id , $item, $error_msg)</div>";
		$item = $this->getItem($id, $item);
		
		$this->checkEditable($item['id'], $item);

		// If this record is the primary record set the checkbox readonly
		$this->setPrimaryReadOnly($item);
		//add any data elements not in item to the item array
		$this->beforeForm($id, $item, $error_msg);

		// Removes permissions fields that the user is not allowed to see and/or edit
		// Either because of permission locking (at a module or contact level)
		// or bcause the user is not a member of the required group to edit a permission field
		$this->handlePermissions($id, $item, $error_msg);
		
		// return parent::main($id, $item, $error_msg);
		return parent::main($item['id'], $item, $error_msg);
	}
	
	/**
	 * runs on submit to validate the data and then update the DB
	 */
	function doSubmit() {
		$this->doSubmitBegin();
		$this->doSubmitPermissions();
		
		if ($this->submit) {
			$this->doSubmitValidate();
			$id = $this->doSubmitProcess();
			$this->doSubmitPrintErrors($id);
		} else if ($this->delete) {
			$this->doSubmitDelete();
		}
		
		$this->doSubmitEnd($id);
		$this->doRedirect($id);
		$this->redirectBack($id);
		
		return $id;
	}

	/**
	 * redirects the user to the page based on redirect option 
	 * @param integer $id optional the item id
	 * @author Syed
	 */
	function doRedirect($id='') {
		switch ($this->form_values['redirect']) {
			case 'list':
				$this->redirectToList($id);
				break;
			case 'View':
				$this->redirectToView($id);
				break;
			case 'Add':
				$this->redirectToEdit($id);
				break;
			case 'Contact': // Phil 20051208 - Note: dont break here incase there is no contact
				$this->redirectToContactView($id); 
			default:
				$this->redirectBack($id);
		}
	}
	
	/**
	 * redirects to the view page
	 * @param integer $id the item id
	 * @author Syed
	 */
	function redirectToView($id) {
		$link_href = $this->getModulePath() . $this->getFileSetHref('view');
		if ($this->form_values['contact_id']) {
			if (!strstr($link_href, '%contact_id%')) {
				$link_href .= '&contact_id=' . $this->form_values;
			}
		}
		$link_href = str_replace("%id%",$id,$link_href);
		header("location:" . $link_href);
		exit;
	}
	
	/**
	 * redirects to the edit page
	 * @author Syed
	 */
	function redirectToEdit() {
		$link_href = $this->getFileSetHref($this->edit_link);
		$link_href = $this->getModulePath() . $link_href;
		$link_href = str_replace("%id%",$id,$link_href);
		$link_href = str_replace("%contact_id%",$this->form_values['contact_id'],$link_href);
		header("location:" . $link_href);
		exit;
	}
	
	/**
	 * Redirects to the contact view page
	 * @author Phil Calder
	 */
	function redirectToContactView() {
		if (isset($this->form_values['contact_id']) && $this->form_values['contact_id']) {
			global $SERVICES;
			$SERVICES->getService('Modules');
			$SERVICES->getService('Config');
			$contact_mod =& $SERVICES->modules->getModule('Contact');
			$link_href = $SERVICES->config->lookup('modules.Contact.file_set.item.functions.contact_view');
			$link_href = $contact_mod->getModulePath() . $link_href;
			$link_href = str_replace("%id%",$this->form_values['contact_id'],$link_href);
			$link_href = str_replace("%contact_id%",'',$link_href);
			// print $link_href;
			header("location:" . $link_href);
			exit;
		}
	}
	
	function getLinkReturnPage() {
		global $L;
		$this->output->getLinkHref( array('href' => $this->getReturnPage(), 'text' => $L['Back']) ) ;
	}
	
	/**
	 * a blank function at the start of $this->doSubmit()
	 */
	function doSubmitBegin() {}
	
	/**
	 * check if the user can edit the item
	 */
	function doSubmitPermissions() {
		bcrm_unset($this->item, '_permission_controls'); // Phil 20050712 @see addPermissionControls
		$this->checkEditable($this->item['id'], $this->item);
	}
	
	/**
	 * validate the item
	 */
	function doSubmitValidate() {
		if ($error = $this->validateFields($this->item)) {
			// $this->main( $error['item']['id'], $error['item'], $error['error_msg']);
			// Phil 20061122 Call doSubmitValidateErrorHandler instead of main. 
			// Allows for custom handling in child classes.
			$this->doSubmitValidateErrorHandler( $error['item']['id'], $error['item'], $error['error_msg']);
			// exit(); //
		}
		$this->doSubmitRemoveReadOnly();
	}
	
	/**
	 * removes the read only fields
	 */
	function doSubmitRemoveReadOnly() {
		$separator = "\n\n";
		
		// Phil 20070308 Remove any fields beginning with '__tmp'
		$prefix = '__tmp';
		if (is_array($this->item)) {
			foreach ($this->item as $k => $v) {
				if (substr($k, 0, strlen($prefix)) == $prefix) { 
					// Phil 20070308 Remove any field starting with __tmp
					bcrm_unset($this->item, $k);
				} 
			}
		}
		// --
		
		
		if ($this->item['id']) {
			foreach ($fields = $this->db_functions->getDataDefinitionFields() as $k => $v) {
				// print '<div>doSubmitRemoveReadOnly</div>'; print_r($v); print '<hr>';
				if ($v['read_only']) {
					// Phil 20070308, see WebMATDatabaseTable::getMetaFields, WebMATEdit::handleMetaFields
					// Dont unset the read only field 'last_update'
					if (! ($v['field_key'] == 'last_update') ) { 
						// print '<div>unsetting read_only ' . $v['field_key'] . '</div>';
						bcrm_unset($this->item, $v['field_key']);
					}
				} else if ( ($v['type'] == 'long_text') && $v['append_only']) { // Phil 20060608
					// Get the original value for this field (use db_functions, not this class)
					$original_item = $item = $this->db_functions->getItem($this->item['id']);
					if (is_array($original_item) && isset($original_item[$v['field_key']]) && $original_item[$v['field_key']]) {
						// print '<div>original_item[' . $v['field_key'] . ']: {' . $original_item[$v['field_key']] . '}</div>';
						
						// See the Module Editor. Any new cases here will need to be matched on the edit field form.
						switch ($v['append_only']) {
							case 2:
								$separator = "\n\n--------\n";
							break;
							case 3:
								global $SERVICES, $USER;
								$date_time = date("Y-m-d H:i", $SERVICES->offsetTimeZoneForRead(time()));
								$separator = "\n\n[" . $USER['username'] . "]\n";
							break;
							case 4:
								global $SERVICES, $USER;
								$date_time = date("Y-m-d H:i", $SERVICES->offsetTimeZoneForRead(time()));
								$separator = "\n\n[" . $USER['username'] . " " . $date_time . "]\n";
							break;
							case 1:
							default:
								$separator = "\n\n";
							break;
						}
						
						$this->item[$v['field_key']] = $original_item[$v['field_key']] . $separator . $this->item[$v['field_key']];
					}
				}
				// print '<div>new value [' . $v['field_key'] . ']: {' . $this->item[$v['field_key']] . '}</div>';
			}
		}
		// print '<hr>'; exit();
	}
	
	/**
	 * processes the item and returns the id
	 *
	 * @return integer id the item that was created/updated
	 */
	function doSubmitProcess() {
		return $this->setItem();
	}
	
	/**
	 * print an insert time error if there is one
	 *
	 * @param integer $id the item id
	 */
	function doSubmitPrintErrors($id) {
		if ($error = $this->getErrorMessage()) {
			global $L;
			quickPage( $L[$error] . '<br><br> ' . $this->getLinkReturnPage() );
			exit();
		}
	}
	
	/**
	 * deletes the item.
	 */
	function doSubmitDelete() {
		$this->item = $this->db_functions->getItem($this->delete);
		if ($this->isDeletable($this->delete, $this->item)) { // Phil 20060608
			$this->db_functions->deleteItem($this->delete);
			$this->afterDelete($this->item);
		}
	}
	
	/**
	 * a blank function at the end of $this->doSubmit()
	 */
	function doSubmitEnd($id) {}
	
	/**
	 * Validates the items array using the validateFields page function
	 *
	 * @param array $item the item array
	 * @return array containing the item and the error message if an error message is present
	 */
	function validateFields(& $item) {
		// print_r($item);
		$fields = $this->dd->getDataStructureWithData($item);	//get field definitions
		// print_r($fields);
		$this->output->validateFields($item, $fields, $error_msg); //validate $item based on the definitions in $fields; $item values will get updated
		// print_r($item);
		if (is_array($error_msg)) {		//if server-side validation errors, return to form
			return array('item' => $item, 'error_msg' => $error_msg);
		}
	}
	
	/**
	 * Writes the item to the database
	 *
	 * @param array $item the item array
	 * @return array containing the item id and the error message(language key) if an error message is present
	 */
	 function setItem($item = false) {
		if (!$item) {
			$item = $this->getItemVariable(); // $this->item;
		}
		// print '<p>setItem getItemVariable</p>'; print_r($item); print '<hr>';

		$this->beforeDBWrite($item, $fields, $error_msg);  //passed by reference
		$updated = '';
		$item = $this->removeForeignFields($item);
		$updated = $this->db_functions->setItem($item); //To add more cases you need to update setItem
		// DEBUG // $this->db_functions->db->printError();
		switch ($updated) { // Error catching for updates
			case 'Deleted': //If item has been deleted while editing
			case 'Updated': //If item has been changed while editing
				global $L;
				$this->setErrorMessage('errItem' . $updated);
				return $item['id'];
			break;
			default:
			// Do nothing
			break;
		}

		$this->afterDBWrite($item, $fields, $error_msg, $updated);
		return $updated;
	}
	
	/**
	 * sets the error message variable
	 *
	 * @param string the error message
	 */
	function setErrorMessage($error) {
		$this->error_message = $error;
	}
	
	/**
	 * return the error message if one exists
	 *
	 * @return mixed the error message if one exists or false
	 */
	function getErrorMessage() {
		if ($this->error_message) {
			return $this->error_message;
		}
		return false;
	}
	
	/**
	 * @param integer $contact_id The contact to get the primary record for
	 * @return mixed false or item array
	 * @author Phil Calder
	 * @see WebMATDatabaseTables::getPrimaryRecord
	 */
	function getPrimaryRecordFor($contact_id) {
		return $this->db_functions->getPrimaryRecord($contact_id);
	}
	
	/**
	 * Sets the primary field checkbox to readonly if this item is the current primary
	 */
	function setPrimaryReadOnly($item) {
		$primary_field = $this->db_functions->getPrimaryFieldName();
		if ($item[$primary_field] == 1) {
			$this->dd->changeAttributes(array('field_key' => $primary_field, 'read_only' => 1));
		}
	}
	
	/**
	 * Gets the table links
	 *
	 * Overridden to handle new items and to remove the edit link
	 *
	 * @param array $item the item array
	 * @return array an array of links
	 * @author Phil Calder
	 */
	function getTableLinks($item) {
		
		// Phil 20060310 Edited to handle new items better
		$links = array();
		if ($this->form_values['id']) {	
			$links = parent::getTableLinks($item);
			bcrm_unset($links, 'edit');
		} else {
			$links = array(
				'list' => $this->replaceLinkVariables(false, $item, (array)$this->getListLink($item))
			);
		}
		// print '<div>getTableLinks returning:</div>'; print_r($links); print '<hr>';
		return $links;
	}
	
	/**
	 * An edit page returns an edit form
	 *
	 * @param array $item An item array
	 * @param array $error_msg error message (can be empty)
	 * @return string Content for an edit form
	 * @see getForm
	 * @author Phil Calder
	 */
	function getDataContent($item, $error_msg) {
		return $this->getForm($item, $error_msg);
	}

	/**
	 * gets the form section of the page
	 *
	 * As of 2005-07-28 this function includes the 'nested' structure
	 * which was previously in main(). The old way gave incorrect html with the
	 * form tag between table and row tags.
 	 *
	 * @param array $item the item array
	 * @param array $error_msg the error message array
	 * @return string the content for the form
	 */

	//MCR 2005-07-28
	function getForm($item, $error_msg = false, $table_type = 'editTbl', $is_nested = true ) {  //'viewTbl'
		global $SERVICES;
		
		$action = $SERVICES->getGlobal('php_self') . '?' . $SERVICES->getGlobal('query_string');
		$encoding_type = $this->encodingType();

		// if we're using templates or not
		if ($this->output->getHasTemplate()) {

			return array (
				'beginForm' => $this->output->getBeginForm($action, $this->form_js_validate, $encoding_type),
				'fields' => $this->getFieldRows($item, $error_msg),
				'nextPageRow' => $this->getNextPageRow(),
				'submitRow' => $this->getSubmitRow(),
				'endForm' => $this->output->getEndForm()
				);
		} else {
			
			//build content
			$content = '';
			// if ($is_nested) {
			// 		$content .=	$this->getBeginDataCell(); //$this->output->getBeginRow() . $this->output->getBeginCell(2);
			// }
			$content .= $this->output->getBeginForm($action, $this->form_js_validate, $encoding_type);
			
			if ($this->container_not_table) {
				// $content .=	$this->output->getBeginTable($table_type);
				// $content .= '<tr><td>';
				$content .= $this->getFieldRows($item, $error_msg);
				$content .= '<br><div><br>&nbsp;<table class="editTbl" style="width:500px" border="0" cellpadding="0" cellspacing="0">';
				$content .= $this->getSubmitRow(); // TODO - row not in a table
				$content .= '</table></div>';
				// $content .= '</td></tr>';
				// $content .=	$this->output->getEndTable();
			} else {
				$content .=	$this->output->getBeginTable($table_type);
				$content .= $this->getFieldRows($item, $error_msg);
				$content .= $this->output->getSpacerRow();
				$content .= $this->getSubmitRow();
				$content .=	$this->output->getEndTable();
			}
			
			$content .= $this->output->getEndForm();
			
			// if ($is_nested) {
				// 	$content .=	$this->getEndDataCell();// $this->output->getEndCell() .$this->output->getEndRow();
			// }
			return $content;
		}
	}

	/**
	 * gets the next page row 
	 * Moved to Output class (since it predominantly does output) ~ BB 20050908
	 * @return string the next page row for the form
	 * @author Syed
	 */
	function getNextPageRow() {
		return $this->output->getNextPageRow();
	}
	
	/**
	 * gets the submit row for the form
	 * @return string the submit row for the form
	 */
	function getSubmitRow() {
		
		if (!$this->output->getHasTemplate()) {
			$content = $this->getNextPageRow();
		}
		// Phil 20050712 - @see addPermissionControls
		if ($this->dd->data_dictionary['_permission_controls']) {
			$tag_extras = '
				onClick="
				if (document.forms[\'mainform\'].elements[\'item[_permission_controls]\'].checked == true) {
					document.forms[\'mainform\'].elements[\'item[_permission_controls]\'].checked = false;
				}
				"';
			return $content . $this->output->getSubmitRow($label = 'default', $blank_cells_before_button = 2, $name = 'submit', $tag_extras);
		}
		return $content . $this->output->getSubmitRow();
	}

	/**
	 * @deprecated version 4.150 see {@link getFieldRows}
	 */
	function getHtmlFieldRows($item, $error_msg) {
		return $this->getFieldRows($item, $error_msg);
	}
	
	/** 
	 * Meta fields dont require validation. 
	 *
	 * For new items, meta fields are hidden. Last update is hidden if not set.
	 *
	 * @see getFieldRows
	 * @author Phil Calder
	 * @access private
	 */
	function handleMetaFields($dd_item, $id) {
		if (is_array($dd_item)) {
			$meta_fields = $this->dd->getMetaFields();
			if (is_array($meta_fields) && count($meta_fields)) {
				foreach ($dd_item as $k => $v) {
					if (in_array($v['field_key'], $meta_fields)) {
						if (!$id) {
							$dd_item[$k]['type'] = 'hidden';
							$dd_item[$k]['label'] = '';
						} else if ( ($v['field_key'] == 'last_update') && (in_array('last_update', $meta_fields)) && !$v['value']) {
							// Phil 20070308 Dont display last_update for existing items that have not been updated
							$dd_item[$k]['type'] = 'hidden';
							$dd_item[$k]['label'] = '';
						} 
						if ( ($v['field_key'] == 'last_reach') && (in_array('last_reach', $meta_fields)) && !$v['value']) {
							// Phil 20070314 Dont display last_reach for existing contacts that have not been updated
							$dd_item[$k]['type'] = 'hidden';
							$dd_item[$k]['label'] = '';
						}
						$dd_item[$k]['no_client_validation'] = 1;
					}
				}
			}
		}
		return $dd_item;
	}
	
	/** 
	 * Unless the data dictionary field attribute 'allow_resource' is set, dropdown values for user fields contain standard users only
	 *
	 * @see getFieldRows
	 * @author Phil Calder
	 * @access private
	 */
	function handleUserFields($dd_item, $id) {
		if (!$id) {
			if (is_array($dd_item)) {
				$user_fields = $this->dd->getUserFields();
				// print '<div>user fields:</div>'; print_r($user_fields); print '<hr>';
				if (is_array($user_fields) && count($user_fields)) {
					foreach ($dd_item as $k => $v) {
						if (in_array($v['field_key'], $user_fields)) {
							// print '<div>handling: ' . $k . '</div>'; print_r($v); print '<hr>';
							// TODO: implement __USERSRESOURCES__
							if (!isset($v['allow_resources']) || !$v['allow_resources']) {
								$dd_item[$k]['dropdown_values'] = '__USERS__';
							}
							// $dd_item[$k]['label'] = '';
						}
					}
				}
			}
		}
		return $dd_item;
	}
	
	/** 
	 * Append only fields are not required for validation if this is an existing item
	 *
	 * @see getFieldRows
	 * @author Phil Calder
	 * @access private
	 */
	function handleOtherFields($dd_item, $id) {
		if ($id) {
			if (is_array($dd_item)) {
				foreach ($dd_item as $k => $v) {
					if (isset($v['append_only']) && $v['append_only']) {
						// print '<div>handling: ' . $k . '</div>'; print_r($v); print '<hr>';
						$dd_item[$k]['notnull'] = false;
					}
				}
			}
		}
		return $dd_item;
	}

	/**
	 * Gets content for form fields
	 *
	 * @param array $item the item array
	 * @param array $error_msg the error message array
	 * @return mixed string or array content
	 * @author Phil Calder, Brenton Bartel
	 */
	function getFieldRows($item, $error_msg) {
		
		$dd_item = $this->dd->getDataStructureWithData($item, $this->getPriorityEdit());
		$dd_item = $this->handleMetaFields($dd_item, $item['id']);
		$dd_item = $this->handleUserFields($dd_item, $item['id']);
		$dd_item = $this->handleOtherFields($dd_item, $item['id']);
		
		// print '<div>' . get_class($this) . '::getFieldRows item:</div>'; print_r($item); print '<hr>';
		// print '<div>' . get_class($this) . '::getFieldRows dd_item:</div>'; print_r($dd_item); print '<hr>';	
		if ($this->output->getHasTemplate()) {
			$hidden = array();
			$input = array();
			if (is_array($dd_item)) {
				foreach ($dd_item as $k => $v) {
					$field = $this->getFieldRow($v, $error_msg[$v['field_key']]); // Phil 20050915 using wrapper function now
					// we need to know what array to add it to.
					// based on whether or not there's a label
					if (isset($field['label'])) {
						$input[$v['field_key']] = $field;
					} else {
						$hidden[$v['field_key']] = $field;
					}
				}
			}

			return array(
				'hidden' => $hidden,
				'input' => $input
			);
				
		} else {
			$content = '';
			if (is_array($dd_item)) {
				foreach ($dd_item as $k => $v) {
					$content .= $this->getFieldRow($v, $error_msg[$v['field_key']]); // Phil 20050915 using wrapper function now
				}
			}
			return $content;
		}
		return false;
	}
	
	/**
	 * Wrapper function for {@link Output::getHtmlFieldRow}
	 * @author Phil Calder
	 */
	function getFieldRow($field, $error_msg) {
		return $this->output->getHtmlFieldRow($field, $error_msg);
	}
	
	/**
	 * @deprecated version 4.150 see {@link getFieldRow}
	 */
	function getHtmlFieldRow($field, $error_msg) {
		return $this->getFieldRow($field, $error_msg);
	}

	/**
	 * gets the item from the database
	 *
	 * @param integer $id the it of the item
	 * @param array $item the item array
	 * @return array the item
	 */
	function getItem($id = false, $item = false) {
		$contact_id = $this->form_values['contact_id'];
		if (!$item) {
			$item = $this->form_values['item'];
		}
		// if (!$item) {  //item could be passed back from server-side data validation failure
		// Phil 20060615 - Need to allow for new items where some item values are specified (eg in query str ?id=&item[key]=value
		if (!$item || ($item && !$id)) {  //item could be passed back from server-side data validation failure
			if (!$id) {
				if (!$id = $this->form_values['id']) {
					$id = $this->form_values['duplicate'];
				}
			}
			//if this is an existing item, then get it
			if ($id) {
				//get existing item from db
				
				$item = $this->db_functions->getItem($id);
				
				// Phil, 20070315 - called this AFTER getItem 
				// (ie only one db call because we can pass the item to isDeleted)
				if ($this->db_functions->isDeleted($id, $item)) {
					$this->redirectToList();
				}
				
				if ($this->form_values['duplicate']) {
					bcrm_unset($item, 'id');
					bcrm_unset($item, 'last_update');
					bcrm_unset($item, 'last_reach');
					bcrm_unset($item, 'created');
					bcrm_unset($item, 'active');
				} else {
					$this->markAsSeen($item);
				}

			} else {
				// Phil 20060615
				if ($item) {
					$item['contact_id'] = $contact_id;
				} else {
					$item = array('contact_id' => $contact_id);
				}
				//set dd with default values
				// $item = $this->dd->getDefaultItem(array('contact_id' => $contact_id));
				// Phil 20060530 Calls wrapper function
				// $item = $this->getDefaultItem(array('contact_id' => $contact_id));
				// Phil 20060615
				$item = $this->getDefaultItem($item);
				// print '<div>' . get_class($this) . ' default_item:</div>'; print_r($item); print '<hr>';
			}
		}	
		return $item;
	}
	
	/**
	 * gets default values
	 *
	 * Wrapper function
	 * @author Phil Calder
	 * @param array $default_overrides values that overwrite the data definition defaults.
	 */
	function getDefaultItem($default_overrides = false) {
		// test: //$default_overrides = array('Accounts__item.is_alive' => 1);
		return $this->dd->getDefaultItem($default_overrides);
	}

	/**
	 * Determines whether the item is editable by the current user
	 *
	 * @param integer $id the item id
	 * @param mixed item - item array or false
	 * @return boolean true if editable, false if not (note: new items are editable)
	 * @author Phil Calder
	 */
	function isEditable($id, $item = false){
		// id passed in is for an existing item
		if ($id || ($this->ua->isWebFormUser() && $this->ua->isWebFormUserAllowedToEditOwnContact()) ) {
			if (!$id) { // Phil 20050909
				if ($this->form_values['id']) {
					$id = $this->form_values['id'];
				} else {
					$id = $this->form_values['item']['id'];
				}
			}
			return $this->db_functions->isEditable($id, $item);
		}
		return true; // 20050902
	}
	
	/**
	 * Wrapper function for WebMATDatabaseTables
	 *
	 * @param integer $id the item id
	 * @param mixed item - item array or false
	 * @return boolean true if deletable, false if not (note: new items are deletable)
	 * @author Phil Calder
	 */
	function isDeletable($id, $item = false) {
		return $this->db_functions->isDeletable($id, $item);
	}

	/**
	 * @param integer id The item id
	 * @param mixed Either false, or the item array to pass to isEditable and isViewable
	 * Does not return a value. Displays an error message and halts if the user can not edit the item.
	 */
	function checkEditable($id, $item = false) {

		if (!$this->isEditable($id, $item)) {
			global $L;
			// Phil 20050727
			$content = $L['errNoPermissions'] . ' &nbsp;&nbsp; ' . $this->getLinkReturnPage();
			// print '<div>'  . get_class($this) . '::checkEditable(' . $id . ', ' . ($item === false ? 'false' : 'item') . ') calling isViewable</div>'; print '<hr>'; 
			if ($this->isViewable($id, $item)) {
				if (!$id) {
					$id = $this->form_values['id'];
				}
				if ($id) {
					$view_link = $this->getViewLink($id);
					if (strlen($view_link)) {
						$content .= '<br><br>' . $L['errCanViewNotEdit'] . ' &nbsp;&nbsp; ' . str_replace('%id%', $id, $view_link);
					}
				}
			}

			quickPage($content);
			exit();
		}
	}

	/**
	 * tests to see if a 'file_upload' field is in the DataDictionary
	 *
	 * @return string the encoding type or '' if no 'file_upload' field
	 */
	function encodingType() {
		foreach ($this->dd->getDataDictionary() as $k => $v) {
			if ($v['type'] == 'file_upload') {
				return 'multipart/form-data';
			}
		}
		return '';
	}
	
	/**
	 * Gets the title text for the page
	 *
	 * CHANGED to change the prefix
	 *
	 * @param string $prefix what to prefix the file name with (optional)
	 * @return string with the title
	 */
	function getTitleText($prefix = 'Title_') {
		if (($prefix == 'Title_') && !$this->form_values['id']) {
			$prefix = 'TitleNew_';
		}
		return parent::getTitleText($prefix);
	}

	/**
	 * any processing required before adding to the db
	 *
	 * @param array $item
	 * @param array $fields
	 * @param array $error_msg
	 */
	function beforeDBWrite(&$item, &$fields, &$error_msg) {  //passed by reference
		//any other processing before posting to database
	}

	/**
	 * any processing required after adding to the db
	 *
	 * @param array $item
	 * @param array $fields
	 * @param array $error_msg
	 */
	function afterDBWrite(&$item, &$fields, &$error_msg, $id = false) {  //passed by reference
		//any other processing after posting to database
	}

	/**
	 * Handles permission fields the user can not edit
	 *
	 * Calls permissions methods to remove and/or set permissions fields
	 * as read-only depending on groups the user is a member of
	 * @param integer $id The item id from {@link init()}
	 * @param array $item The item array from {@link init()}
	 * @param array $error_msg The error message from {@link init()}
	 * @see removeLockedPermissions()
	 * @see setPermissionsFieldsReadOnly()
	 * @author Phil Calder
	 */
	function handlePermissions(&$id, &$item, &$error_msg) {
		//remove permissions fields if this item is locked (either in the config, or by the contact attached to it)
		$this->removeLockedPermissions($id, $item, $error_msg);
		$this->setPermissionsFieldsReadOnly($id, $item, $error_msg);
		$this->addPermissionControls($id, $item, $error_msg);
	}

	/**
	 * Current user must be a member of a group higher than the permissions field to be able to change it
	 * ie:
	 * a member of the admin group can edit the owner field, edit group field, view group field
	 * a member of the owner group can edit the edit group field and the view_group_field
	 * a member of the edit group can edit the view_group_field
	 * @author Phil Calder
	 */
	function setPermissionsFieldsReadOnly(&$id, &$item, &$error_msg) {
		global $USER, $SERVICES;

		$exclude_fields = array();

		if ($this->db_functions) {

			$continue = true;

			if ($this->db_functions->isAdmin($item['id'], $item)) {
				$continue = false;
				return;
			} else {
				$exclude_fields[] = 'admin_group_id';
				$exclude_fields[] = 'owner_group_id';
			}

			if ($continue) {
				if ($this->db_functions->isOwner($item['id'], $item)) {
					$continue = false;
				} else {
					$exclude_fields[] = 'edit_group_id';
				}
			}

			if ($continue) {
				if ($this->db_functions->isEditable($item['id'], $item)) {
					$continue = false;
				} else {
					$exclude_fields[] = 'view_group_id';
				}
			}

			if (count($exclude_fields)) {
				foreach($exclude_fields as $k => $v) {
					$this->dd->setReadOnly($v);
				}
				// foreach($exclude_fields as $k => $v) {
					// bcrm_unset($item, $v);
				// }
			}
		}
	}

	/**
	 * Removes the permissions fields from the item if the permissions are locked to a contact
	 *
	 * Permissions are locked in two cases:
	 * The module permissions are locked (config option)
	 * The contact attached to the item requires permissions to be locked
	 * @author Phil Calder
	 */
	function removeLockedPermissions(&$id, &$item, &$error_msg) {
		global $USER, $SERVICES;
		if ($SERVICES->modules->exists('Contact')) {

			if ($this->module_name != 'Contact') {

				$contact_lock_permissions = false;
				if ($item['contact_id']) {
					$contact_mod =& $SERVICES->modules->getModule('Contact');
					$contact_db_functions =& $contact_mod->getDatabaseTable('item');
					$contact_item = $contact_db_functions->getItem($item['contact_id']);
					$contact_lock_permissions = $contact_item['lock_permissions'];
				}
				if (
					( ($this->getDisplayByGroupFlag() == 2) && $item['contact_id'])
					|| $contact_lock_permissions
				) {
					$this->dd->setDictionaryExcludeFields(
						array(
							'view_group_id',
							'edit_group_id',
							'owner_group_id',
							'admin_group_id'
						)
					);
					bcrm_unset($item, 'view_group_id');
					bcrm_unset($item, 'edit_group_id');
					bcrm_unset($item, 'owner_group_id');
					bcrm_unset($item, 'admin_group_id');
				}
			} else {
				$contact_mod =& $SERVICES->modules->getModule('Contact');
				$contact_db_functions =& $contact_mod->getDatabaseTable('item');
				if (!($contact_db_functions->isOwner($item['id']))) {
					$this->dd->setDictionaryExcludeFields(
						array('lock_permissions')
					);
					bcrm_unset($item, 'lock_permissions');
				}
				$this->dd->setReadOnly('display_name');
			}

		}
	}
	
	/**
	 * returns the display by group flag for the current file set
	 * @return integer Display by group flag (0,1,2)
	 * @author Phil Calder 20051005
	 * @see WebMATDatabaseTable::getDisplayByGroupFlag
	 */
	function getDisplayByGroupFlag() {
		global $SERVICES;
		if (!isset($this->display_by_group_flag)) {
			$this->display_by_group_flag = $SERVICES->config->lookup($this->db_functions->getFilesetConfigPath() . 'display_by_group');
		}	
		return $this->display_by_group_flag;
	}
	
	/**
	 * @deprecated
	 * @see addPermissionControls
	 */
	function addMakePermissionsPrivate(&$id, &$item, &$error_msg) {
		return false;
	}

	/**
	 * Add a field to the data definition which allows the user
	 * to make the item 'private' according to that users preferences
	 * This can only be done if the user has at least 'owner' rights
	 * and the permissions fields are not read_only.
	 *
	 * @author Phil Calder
	 */
	function addPermissionControls(&$id, &$item, &$error_msg) {
		
		if (is_object($this->db_functions) && $this->db_functions->isOwner($item['id'], $item)) {

			global $USER, $SERVICES;
			$SERVICES->getService('Preferences');
			
			// Phil 20060209 Added flag to return if there are no permissions fields
			$has_perms = false;
			
			// Phil 20060201 Added code to change priority_edit

			$prefs = array();
			// print_r($this->dd->data_dictionary);
			if (is_array($this->dd->data_dictionary['lock_permissions'])) {
				$this->dd->changeAttributes(array('lock_permissions' => 'view_group_id', 'priority_edit' => '993'));
			}
			
			if ($this->dd->data_dictionary['view_group_id'] && is_array($this->dd->data_dictionary['view_group_id'])) {
				$has_perms = true;
				$prefs['default']['view_group_id'] =  $this->db_functions->replacePlaceHolders($SERVICES->preferences->getUserPref('frontDefaultGroupView', 'private_group_id'), true);
				$prefs['personal']['view_group_id'] =  $this->db_functions->replacePlaceHolders($SERVICES->preferences->getUserPref('frontPrivateGroupView', 'private_group_id'), true);
				$prefs['public']['view_group_id'] =  $this->db_functions->replacePlaceHolders($SERVICES->preferences->getUserPref('frontPublicGroupView', 'everyone_group_id'), true);
				
				$this->dd->changeAttributes(array('field_key' => 'view_group_id', 'priority_edit' => '994'));
			}
			
			if ($this->dd->data_dictionary['edit_group_id'] && is_array($this->dd->data_dictionary['edit_group_id'])) {
				$has_perms = true;
				$prefs['default']['edit_group_id'] =  $this->db_functions->replacePlaceHolders($SERVICES->preferences->getUserPref('frontDefaultGroupEdit', 'private_group_id'), true);
				$prefs['personal']['edit_group_id'] =  $this->db_functions->replacePlaceHolders($SERVICES->preferences->getUserPref('frontPrivateGroupEdit', 'private_group_id'), true);
				$prefs['public']['edit_group_id'] =  $this->db_functions->replacePlaceHolders($SERVICES->preferences->getUserPref('frontPublicGroupEdit', 'everyone_group_id'), true);
				$this->dd->changeAttributes(array('field_key' => 'edit_group_id', 'priority_edit' => '995'));
			}
			if ($this->dd->data_dictionary['assigned_to_group_id'] && is_array($this->dd->data_dictionary['assigned_to_group_id'])) {
				// $has_perms = true; // Not for assigned to
				$prefs['default']['assigned_to_group_id'] =  $this->db_functions->replacePlaceHolders($SERVICES->preferences->getUserPref('frontDefaultGroupAssignedTo', 'private_group_id'), true);
				$prefs['personal']['assigned_to_group_id'] =  $this->db_functions->replacePlaceHolders($SERVICES->preferences->getUserPref('frontPrivateGroupAssignedTo', 'private_group_id'), true);
				$prefs['public']['assigned_to_group_id'] =  $this->db_functions->replacePlaceHolders($SERVICES->preferences->getUserPref('frontPublicGroupAssignedTo', 'everyone_group_id'), true);
				// Dont include assigned to in the permissions group (ie dont change priority_edit)
			}
			if ($this->db_functions->isAdmin($item['id'], $item)) {
				if ($this->dd->data_dictionary['owner_group_id'] && is_array($this->dd->data_dictionary['owner_group_id'])) {
					$has_perms = true;
					$prefs['default']['owner_group_id'] = $this->db_functions->replacePlaceHolders($SERVICES->preferences->getUserPref('frontDefaultGroupOwner', 'private_group_id'), true);
					$prefs['personal']['owner_group_id'] = $this->db_functions->replacePlaceHolders($SERVICES->preferences->getUserPref('frontPrivateGroupOwner', 'private_group_id'), true);
					$prefs['public']['owner_group_id'] = $this->db_functions->replacePlaceHolders($SERVICES->preferences->getUserPref('frontPublicGroupOwner', 'private_group_id'), true);
					$this->dd->changeAttributes(array('field_key' => 'owner_group_id', 'priority_edit' => '996'));
				}
				if ($this->dd->data_dictionary['admin_group_id'] && is_array($this->dd->data_dictionary['admin_group_id'])) {
					$has_perms = true;
					$prefs['default']['admin_group_id'] = $this->db_functions->replacePlaceHolders($SERVICES->preferences->getUserPref('frontDefaultGroupAdmin', 'administrators_group_id'), true);
					$prefs['personal']['admin_group_id'] = $this->db_functions->replacePlaceHolders($SERVICES->preferences->getUserPref('frontPrivateGroupAdmin', 'private_group_id'), true);
					$prefs['public']['admin_group_id'] = $this->db_functions->replacePlaceHolders($SERVICES->preferences->getUserPref('frontPublicGroupAdmin', 'administrators_group_id'), true);
					$this->dd->changeAttributes(array('field_key' => 'admin_group_id', 'priority_edit' => '997'));
				}
			}
			// print_r($prefs); print '<hr>';
			
			if (!$has_perms) {
				return;
			}
			
			$prefs_str = '';
			$prefs_str .= 'defaultView = '       . (isset($prefs['default']['view_group_id']) ? $prefs['default']['view_group_id'] : '""') .';
				';
			$prefs_str .= 'defaultEdit = '       . (isset($prefs['default']['edit_group_id']) ? $prefs['default']['edit_group_id'] : '""') .';
				';
			$prefs_str .= 'defaultAssignedTo = ' . (isset($prefs['default']['assignedto_group_id']) ? $prefs['default']['assignedto_group_id'] : '""') .';
				';
			$prefs_str .= 'defaultOwner = '      . (isset($prefs['default']['owner_group_id']) ? $prefs['default']['owner_group_id'] : '""') .';
				';
			$prefs_str .= 'defaultAdmin = '      . (isset($prefs['default']['admin_group_id']) ? $prefs['default']['admin_group_id'] : '""') .';
				';
				
			$prefs_str .= 'personalView = '       . (isset($prefs['personal']['view_group_id']) ? $prefs['personal']['view_group_id'] : '""') .';
				';
			$prefs_str .= 'personalEdit = '       . (isset($prefs['personal']['edit_group_id']) ? $prefs['personal']['edit_group_id'] : '""') .';
				';
			$prefs_str .= 'personalAssignedTo = ' . (isset($prefs['personal']['assignedto_group_id']) ? $prefs['personal']['assignedto_group_id'] : '""') .';
				';
			$prefs_str .= 'personalOwner = '      . (isset($prefs['personal']['owner_group_id']) ? $prefs['personal']['owner_group_id'] : '""') .';
				';
			$prefs_str .= 'personalAdmin = '      . (isset($prefs['personal']['admin_group_id']) ? $prefs['personal']['admin_group_id'] : '""') .';
				';
			
			$prefs_str .= 'publicView = '       . (isset($prefs['public']['view_group_id']) ? $prefs['public']['view_group_id'] : '""') .';
				';
			$prefs_str .= 'publicEdit = '       . (isset($prefs['public']['edit_group_id']) ? $prefs['public']['edit_group_id'] : '""') .';
				';
			$prefs_str .= 'publicAssignedTo = ' . (isset($prefs['public']['assignedto_group_id']) ? $prefs['public']['assignedto_group_id'] : '""') .';
				';
			$prefs_str .= 'publicOwner = '      . (isset($prefs['public']['owner_group_id']) ? $prefs['public']['owner_group_id'] : '""') .';
				';
			$prefs_str .= 'publicAdmin = '      . (isset($prefs['public']['admin_group_id']) ? $prefs['public']['admin_group_id'] : '""') .';
				';
				
			$prefs_str .= 'otherView = '       . (isset($item['view_group_id']) ? $item['view_group_id'] : '""') .';
				';
			$prefs_str .= 'otherEdit = '       . (isset($item['edit_group_id']) ? $item['edit_group_id'] : '""') .';
				';
			$prefs_str .= 'otherAssignedTo = ' . (isset($item['assignedto_group_id']) ? $item['assignedto_group_id'] : '""') .';
				';
			$prefs_str .= 'otherOwner = '      . (isset($item['owner_group_id']) ? $item['owner_group_id'] : '""') .';
				';
			$prefs_str .= 'otherAdmin = '      . (isset($item['admin_group_id']) ? $item['admin_group_id'] : '""') .';
				';
			
			$handle_permissions_str = '
			<script type="text/javascript">
			function handlePermissions() {
				selected_option = -1;
				
				' . $prefs_str . '
				
				for (i=document.forms[\'mainform\'].elements[\'item[_permission_controls]\'].length-1; i > -1; i--) {
					if (document.forms[\'mainform\'].elements[\'item[_permission_controls]\'][i].checked) {
						selected_option = i;
						selected_value  = document.forms[\'mainform\'].elements[\'item[_permission_controls]\'][i].value
						if (selected_value == "default") {
							if (document.forms[\'mainform\'].elements[\'item[view_group_id]\']) {
								document.forms[\'mainform\'].elements[\'item[view_group_id]\'].value	= defaultView;
							}
							if (document.forms[\'mainform\'].elements[\'item[edit_group_id]\']) {
								document.forms[\'mainform\'].elements[\'item[edit_group_id]\'].value	= defaultEdit;
							}
							if (document.forms[\'mainform\'].elements[\'item[assignedto_group_id]\']) {
								document.forms[\'mainform\'].elements[\'item[assignedto_group_id]\'].value	= defaultAssignedTo;
							}
							if (document.forms[\'mainform\'].elements[\'item[owner_group_id]\']) {
								document.forms[\'mainform\'].elements[\'item[owner_group_id]\'].value	= defaultOwner;
							}
							if (document.forms[\'mainform\'].elements[\'item[admin_group_id]\']) {
								document.forms[\'mainform\'].elements[\'item[admin_group_id]\'].value	= defaultAdmin;
							}
						} else if (selected_value == "personal") {
							if (document.forms[\'mainform\'].elements[\'item[view_group_id]\']) {
								document.forms[\'mainform\'].elements[\'item[view_group_id]\'].value	= personalView;
							}
							if (document.forms[\'mainform\'].elements[\'item[edit_group_id]\']) {
								document.forms[\'mainform\'].elements[\'item[edit_group_id]\'].value	= personalEdit;
							}
							if (document.forms[\'mainform\'].elements[\'item[assignedto_group_id]\']) {
								document.forms[\'mainform\'].elements[\'item[assignedto_group_id]\'].value	= personalAssignedTo;
							}
							if (document.forms[\'mainform\'].elements[\'item[owner_group_id]\']) {
								document.forms[\'mainform\'].elements[\'item[owner_group_id]\'].value	= personalOwner;
							}
							if (document.forms[\'mainform\'].elements[\'item[admin_group_id]\']) {
								document.forms[\'mainform\'].elements[\'item[admin_group_id]\'].value	= personalAdmin;
							}
						} else if (selected_value == "public") {
							if (document.forms[\'mainform\'].elements[\'item[view_group_id]\']) {
								document.forms[\'mainform\'].elements[\'item[view_group_id]\'].value	= publicView;
							}
							if (document.forms[\'mainform\'].elements[\'item[edit_group_id]\']) {
								document.forms[\'mainform\'].elements[\'item[edit_group_id]\'].value	= publicEdit;
							}
							if (document.forms[\'mainform\'].elements[\'item[assignedto_group_id]\']) {
								document.forms[\'mainform\'].elements[\'item[assignedto_group_id]\'].value	= publicAssignedTo;
							}
							if (document.forms[\'mainform\'].elements[\'item[owner_group_id]\']) {
								document.forms[\'mainform\'].elements[\'item[owner_group_id]\'].value	= publicOwner;
							}
							if (document.forms[\'mainform\'].elements[\'item[admin_group_id]\']) {
								document.forms[\'mainform\'].elements[\'item[admin_group_id]\'].value	= publicAdmin;
							}
						} else if (selected_value == "other") {
							if (document.forms[\'mainform\'].elements[\'item[view_group_id]\']) {
								document.forms[\'mainform\'].elements[\'item[view_group_id]\'].value	= otherView;
							}
							if (document.forms[\'mainform\'].elements[\'item[edit_group_id]\']) {
								document.forms[\'mainform\'].elements[\'item[edit_group_id]\'].value	= otherEdit;
							}
							if (document.forms[\'mainform\'].elements[\'item[assignedto_group_id]\']) {
								document.forms[\'mainform\'].elements[\'item[assignedto_group_id]\'].value	= otherAssignedTo;
							}
							if (document.forms[\'mainform\'].elements[\'item[owner_group_id]\']) {
								document.forms[\'mainform\'].elements[\'item[owner_group_id]\'].value	= otherOwner;
							}
							if (document.forms[\'mainform\'].elements[\'item[admin_group_id]\']) {
								document.forms[\'mainform\'].elements[\'item[admin_group_id]\'].value	= otherAdmin;
							}
						}
						break;
					}
				}
			}
			</script>
			';
			$this->output->addToHead($handle_permissions_str);

			$tag_extras_string = ' onChange="handlePermissions();" ';
			
			// Decide which radio button should be selected (default 'other')
			$item_permissions = array();
			if (isset($item['view_group_id'])) {
				$item_permissions['view_group_id'] = $item['view_group_id'];
			}
			if (isset($item['edit_group_id'])) {
				$item_permissions['edit_group_id'] = $item['edit_group_id'];
			}
			if (isset($item['assignedto_group_id'])) {
				$item_permissions['assignedto_group_id'] = $item['assignedto_group_id'];
			}
			if (isset($item['owner_group_id'])) {
				$item_permissions['owner_group_id'] = $item['owner_group_id'];
			}
			if (isset($item['admin_group_id'])) {
				$item_permissions['admin_group_id'] = $item['admin_group_id'];
			}
			
			$match = false;
			foreach ($prefs as $pref_k => $pref_v) {
				foreach ($item_permissions as $item_k => $item_v) {
					if (! ($item_v == $pref_v[$item_k])) {
						$match = false;
						break;
					} else {
						$match = $pref_k;
					}
				}
				if ($match) {
					break;
				}
			}
			
			if ($match) {
				$value = $match;
				$item['_permission_controls'] = $value;
			} else {
				$value = '';
			}
			
			global $L;
			$this->dd->data_dictionary['_permission_controls'] = array(
				'field_key' => '_permission_controls',
				'label' => $L['LabelPermissions'],
				'explanation' => $L['LabelPermissionsExplanation'],
				'type' => 'radio_h', 
				'dropdown_values' => array(
					'default'=> $L['LabelPermissionsDefault'], 
					'personal'=> $L['LabelPermissionsPersonal'], 
					'public'=> $L['LabelPermissionsPublic'], 
					'other'=> $L['LabelPermissionsOther']
				), 
				'value' => '"'.$value .'"', // Phil 20060202 use item[_permission_controls = $value (see above)
				'field_priority' => 993,
				'priority_edit' => 993, 
				'tag_extras' => ' ' . $tag_extras_string
			);
			
			// Wrap our permissions fields in a tbody for show/hide
			// todo: initial show/hide status could be user pref based
			if ($this->dd->data_dictionary['view_group_id'] && $this->dd->data_dictionary['admin_group_id']) {
				$tbody_open = '<tbody id="permissions" style="display:none">';
				$tbody_close = '</tbody>';
				$controls = '<span onClick="showPermissions(this);" class="shower"></span>';
				if ($this->dd->data_dictionary['lock_permissions']) {
					$this->dd->data_dictionary['lock_permissions']['additional_output']['before'] = $tbody_open;
				} else {
					$this->dd->data_dictionary['view_group_id']['additional_output']['before'] = $tbody_open;
				
				}
				$this->dd->data_dictionary['admin_group_id']['additional_output']['after'] = $tbody_close;
				$this->dd->data_dictionary['_permission_controls']['label'] = $controls . $this->dd->data_dictionary['_permission_controls']['label'];
			} 
		}
	}

	/**
	 * any processing required before adding to the form is created
	 *
	 * Override this method to add functionality
	 */
	function beforeForm(&$id, &$item, &$error_msg) {}

	/**
	 * adds links below the menu row
	 *
	 * @param array an array with the link info 
	 * (with the keys 'href', 'text' , 'title' and 'tag_extras')
	 */
	function addSecondMenuLinks($links) {
		if (!$this->second_menu_links) {
			$this->second_menu_links = array();
		}
		if (isset($links['href'])) {
			$this->second_menu_links[] = $this->output->getLinkHref($links);
		} else {
			if (is_array($links)) {
				foreach ($links as $k => $v) {
					$this->second_menu_links[] = $this->output->getLinkHref($v);
				}
			}
		}
	}

	function afterDelete($item) {}
	
	/**
	 * No new link if this is a new item
	 *
	 * @author Phil Calder
	 */
	function getNewLink() {
		if (!isset($this->form_values['id']) || !$this->form_values['id']) {
			return false;
		}
		return parent::getNewLink();
	}
	
	/**
	 * Returns the help link 
	 * 
	 * Overridden if this is a new item
	 * @return mixed (nothing) or the help link, or -1
	 */
	function getHelpLink() {
		if (!isset($this->form_values['id']) || !$this->form_values['id']) {
			// New item
			if ($help_link = $this->output->getHelpId()) {
				$help_link .= '#new';
				return $help_link;
			}
		}
		return parent::getHelpLink();
	}
}
?>